@@ -26,7 +26,6 @@ from twisted.trial.unittest import TestCase as TwistedTestCase
 
 class AppendCommand(ActionQueueCommand):
     """Simple command which appends a value to a list."""
-    blocked = False
 
     def __init__(self, request_queue, append_to, value):
         """Initialize, remembering list and value."""
@@ -34,15 +33,16 @@ class AppendCommand(ActionQueueCommand):
         self.append_to = append_to
         self.value = value
 
-    def _is_runnable(self):
-        """Returns the opposite of self.blocked."""
-        return not self.blocked
-
     def _run(self):
         """Do the actual append."""
         self.append_to.append(self.value)
         return defer.succeed(None)
 
+    def __repr__(self):
+        """Helper when debugging."""
+        return "<AppendCommand: %s>" % (self.value,)
+    __str__ = __repr__
+
 
 class FakeEventQueue(object):
     """Fake event queue."""
@@ -87,9 +87,9 @@ class CommandTests(TwistedTestCase):
         """Test command execution."""
         out = []
         AppendCommand(self.request_queue,
-                      append_to=out, value="a").start()
+                      append_to=out, value="a").queue()
         AppendCommand(self.request_queue,
-                      append_to=out, value="b").start()
+                      append_to=out, value="b").queue()
         self.request_queue.run()
         self.request_queue.run()
         self.assertEqual(["a", "b"], out)
@@ -101,14 +101,15 @@ class CommandTests(TwistedTestCase):
         cmds = [AppendCommand(self.request_queue, append_to=out, value=v) \
                 for v in ("a", "b", "c")]
         for cmd in cmds:
-            cmd.start()
-        cmds[1].blocked = True
+            cmd.queue()
+        cmds[1].is_runnable = False
         self.request_queue.run()
         self.request_queue.run()
         self.assertEqual(["a", "c"], out)
         self.request_queue.run()
         self.assertEqual(["a", "c"], out)
-        cmds[1].blocked = False
+        cmds[1].is_runnable = True
+        self.request_queue.check_conditions()
         self.request_queue.run()
         self.assertEqual(["a", "c", "b"], out)
 
@@ -118,12 +119,13 @@ class CommandTests(TwistedTestCase):
         cmds = [AppendCommand(self.request_queue, append_to=out, value=v) \
                 for v in ("a", "b", "c")]
         for cmd in cmds:
-            cmd.start()
-        cmds[1].blocked = True
+            cmd.queue()
+        cmds[1].is_runnable = False
         self.request_queue.run()
         self.request_queue.run()
         self.assertEqual(["a"], out)
-        cmds[1].blocked = False
+        cmds[1].is_runnable = True
+        self.request_queue.check_conditions()
         self.request_queue.run()
         self.assertEqual(["a", "b"], out)
         self.request_queue.run()
@@ -134,7 +136,7 @@ class CommandTests(TwistedTestCase):
         fixture = [False]
         cmd = WaitForCondition(self.request_queue,
                                condition=lambda: fixture[0])
-        cmd.start()
+        cmd.queue()
         d = self.request_queue.run()
         d.addCallback(lambda _: fixture.append('x'))
 
