@@ -106,6 +106,19 @@ class WatchTests(BaseEQTestCase):
         self.assertTrue(path_ancestor in self.eq._general_watchs)
         self.assertEqual(self.eq._ancestors_watchs, {})
 
+    def test_rm_watch_not_dir_anymore(self):
+        """Test that a watch can be removed even not having the dir anymore.
+
+        This is the case where the directory is deleted from the filesystem,
+        the watch is automatically removed in pyinotify but we need to take
+        care of it from our own data structures.
+        """
+        not_existing_dir = "not-added-dir"
+        self.eq.inotify_add_watch(not_existing_dir)
+        self.assertTrue(self.eq.inotify_has_watch(not_existing_dir))
+        self.eq.inotify_rm_watch(not_existing_dir)
+        self.assertFalse(self.eq.inotify_has_watch(not_existing_dir))
+
     def test_rm_watch_wrong(self):
         """Test that general watchs can be removed."""
         # add two types of watchs
@@ -116,8 +129,9 @@ class WatchTests(BaseEQTestCase):
         self.eq.inotify_add_watch(path_ancestor)
 
         # remove different stuff
-        self.assertRaises(ValueError,
-                          self.eq.inotify_rm_watch, "not-added-dir")
+        self.eq.inotify_rm_watch("not-added-dir")
+        self.assertTrue(self.log_handler.check_warning('remove', 'watch',
+                                                       'not-added-dir'))
 
     def test_rm_watch_general(self):
         """Test that general watchs can be removed."""
@@ -173,6 +187,109 @@ class WatchTests(BaseEQTestCase):
         self.assertFalse(self.eq.inotify_has_watch(path_ancestor))
 
 
+class WatchManagerTests(BaseEQTestCase):
+    """Test the structures where we have the path/watch."""
+
+    timeout = 3
+
+    def setUp(self):
+        """Set up."""
+        BaseEQTestCase.setUp(self)
+        self.deferred = deferred = defer.Deferred()
+
+        class HitMe(object):
+            # class-closure, cannot use self, pylint: disable-msg=E0213
+            def handle_default(*a):
+                    deferred.callback(True)
+
+        self.eq.subscribe(HitMe())
+
+    def test_watch_updated_when_deleting_dir(self):
+        """Internal data structures are fixed when deleting the dir."""
+        path = os.path.join(self.root_dir, "path")
+        os.mkdir(path)
+        self.eq.inotify_add_watch(self.root_dir)
+        self.eq.inotify_add_watch(path)
+
+        # we have the watch, remove the dir, the watch should be gone
+        self.assertTrue(self.eq.inotify_has_watch(path))
+        os.rmdir(path)
+
+        def check(_):
+            """Check state after the event."""
+            self.assertFalse(self.eq.inotify_has_watch(path))
+
+        self.deferred.addCallback(check)
+        return self.deferred
+
+    def test_watch_updated_when_renaming_dir(self):
+        """Internal data structures are fixed when renaming the dir."""
+        path1 = os.path.join(self.root_dir, "path1")
+        path2 = os.path.join(self.root_dir, "path2")
+        os.mkdir(path1)
+        self.eq.inotify_add_watch(self.root_dir)
+        self.eq.inotify_add_watch(path1)
+
+        # we have the watch, rename the dir, new name should have the watch,
+        # old name should not
+        self.assertTrue(self.eq.inotify_has_watch(path1))
+        os.rename(path1, path2)
+
+        def check(_):
+            """Check state after the event."""
+            self.assertTrue(self.eq.inotify_has_watch(path2))
+            self.assertFalse(self.eq.inotify_has_watch(path1))
+
+        self.deferred.addCallback(check)
+        return self.deferred
+
+    def test_watch_updated_when_movingout_dir(self):
+        """Internal data structures are fixed when moving out the dir."""
+        notu1 = os.path.join(self.root_dir, "notu1")
+        path1 = os.path.join(self.root_dir, "path1")
+        path2 = os.path.join(notu1, "path2")
+        os.mkdir(notu1)
+        os.mkdir(path1)
+        self.eq.inotify_add_watch(self.root_dir)
+        self.eq.inotify_add_watch(path1)
+
+        # we have the watch, move it outside watched structure, no more watches
+        self.assertTrue(self.eq.inotify_has_watch(path1))
+        os.rename(path1, path2)
+
+        def check(_):
+            """Check state after the event."""
+            self.assertFalse(self.eq.inotify_has_watch(path1))
+            self.assertFalse(self.eq.inotify_has_watch(path2))
+
+        self.deferred.addCallback(check)
+        return self.deferred
+
+    def test_fix_path_not_there(self):
+        """Try to fix path but it's not there."""
+        self.eq._general_watchs = {}
+        self.eq._ancestors_watchs = {}
+        self.eq.inotify_watch_fix("not-there", "new-one")
+        self.assertTrue(self.log_handler.check_warning("Tried to fix",
+                                                       "not-there"))
+
+    def test_fix_path_general(self):
+        """Fixing path in general watches."""
+        self.eq._general_watchs = {'/path1/foo': 1, '/other': 2}
+        self.eq._ancestors_watchs = {'/foo': 3}
+        self.eq.inotify_watch_fix('/path1/foo', '/path1/new')
+        self.assertEqual(self.eq._general_watchs, {'/path1/new': 1, '/other': 2})
+        self.assertEqual(self.eq._ancestors_watchs, {'/foo': 3})
+
+    def test_fix_path_ancestors(self):
+        """Fixing path in ancestors watches."""
+        self.eq._general_watchs = {'/bar': 3}
+        self.eq._ancestors_watchs = {'/oth': 1, '/other': 2}
+        self.eq.inotify_watch_fix('/oth', '/baz')
+        self.assertEqual(self.eq._general_watchs, {'/bar': 3})
+        self.assertEqual(self.eq._ancestors_watchs, {'/baz': 1, '/other': 2})
+
+
 class DynamicHitMe(object):
     """Helper class to test a sequence of signals."""
 
@@ -223,11 +340,6 @@ class BaseTwisted(BaseEQTestCase):
         """Setup the test."""
         BaseEQTestCase.setUp(self)
 
-        # add a Memento handler to the logger
-        self.handler = testcase.MementoHandler()
-        self.handler.setLevel(logging.INFO)
-        self.eq.log.addHandler(self.handler)
-
         # create the deferred for the tests
         self._deferred = defer.Deferred()
 
@@ -375,11 +487,7 @@ class SignalingTests(BaseTwisted):
                     self.finished_error("received a wrong path")
                     return
 
-                check = len(self.handler.records) == 1 and \
-                        'FS_FILE_DELETE' in self.handler.records[0].message and\
-                        testfile in self.handler.records[0].message and \
-                        self.handler.records[0].levelno == logging.INFO
-                if not check:
+                if not self.log_handler.check_info('FS_FILE_DELETE', testfile):
                     self.finished_error("FS_FILE_DELETE must appear in INFO.")
                     return
 
@@ -405,14 +513,13 @@ class SignalingTests(BaseTwisted):
                     self.finished_error("received a wrong path")
                     return
 
-                check = len(self.handler.records) == 1 and \
-                        'FS_DIR_DELETE' in self.handler.records[0].message and\
-                        testdir in self.handler.records[0].message and \
-                        self.handler.records[0].levelno == logging.INFO
-                if not check:
+                if not self.log_handler.check_info('FS_DIR_DELETE', testdir):
                     self.finished_error("FS_DIR_DELETE must appear in INFO.")
                     return
 
+                # file deletion should remove its watch
+                self.assertFalse(self.eq.inotify_has_watch(testdir))
+
                 self.finished_ok()
 
         self.eq.inotify_add_watch(self.root_dir)
