@@ -31,7 +31,7 @@ from twisted.internet import defer, reactor
 
 from contrib.testing import testcase
 from ubuntuone.syncdaemon import hash_queue
-from ubuntuone.storageprotocol.hash import \
+from ubuntuone.storageprotocol.content_hash import \
             content_hash_factory, crc32
 
 
@@ -74,6 +74,50 @@ class HasherTests(testcase.BaseTwistedTestCase):
         self.assertFalse(hasher.isAlive())
         hasher.join(timeout=5)
 
+    @defer.inlineCallbacks
+    def test_called_back_log_ok(self):
+        """Tests that the hasher produces correct info."""
+        # create the hasher
+        mark = object()
+        queue = hash_queue.UniqueQueue()
+        d = defer.Deferred()
+        class Helper(object):
+            """Helper class."""
+            def push(self, *args):
+                """Callback."""
+                d.callback(args)
+        receiver = Helper()
+        hasher = hash_queue._Hasher(queue, mark, receiver)
+        hasher.start()
+
+        # log config
+        handler = testcase.MementoHandler()
+        handler.setLevel(logging.DEBUG)
+        hasher.logger.addHandler(handler)
+
+        # send what to hash
+        testfile = os.path.join(self.test_dir, "testfile")
+        with open(testfile, "w") as fh:
+            fh.write("foobar")
+        queue.put((testfile, "mdid"))
+
+        # wait event and stop hasher
+        yield d
+        hasher.stop()
+        hasher.join(timeout=5)
+
+        # check log
+        log_msg = [r.message for r in handler.records
+                   if "path hash pushed" in r.message][0]
+        self.assertTrue("path" in log_msg)
+        self.assertTrue("hash" in log_msg)
+        self.assertTrue("crc" in log_msg)
+        self.assertTrue("size" in log_msg)
+        self.assertTrue("st_ino" in log_msg)
+        self.assertTrue("st_size" in log_msg)
+        self.assertTrue("st_mtime" in log_msg)
+        hasher.logger.removeHandler(handler)
+
     def test_called_back_ok(self):
         """Tests that the hasher produces correct info."""
         # create the hasher
