@@ -19,19 +19,22 @@
 
 from __future__ import with_statement
 
-import unittest
+import logging
 import os
 import shutil
+import unittest
+import uuid
 
 from twisted.internet import defer, reactor
 
 from contrib.testing import testcase
 from ubuntuone.syncdaemon.local_rescan import LocalRescan
+from ubuntuone.syncdaemon.marker import MDMarker
 from ubuntuone.syncdaemon import (
     event_queue, filesystem_manager, volume_manager
 )
 from ubuntuone.storageprotocol import (
-    hash as storage_hash, volumes
+    content_hash as storage_hash, volumes
 )
 
 
@@ -58,11 +61,27 @@ class FakeAQ(object):
     """Fake AQ"""
     def __init__(self):
         self.unlinked = []
+        self.moved = []
+        self.uploaded = None
+        self.downloaded = None
+
+    def upload(self, *args):
+        """Store stuff as uploaded."""
+        self.uploaded = args
+
+    def download(self, *args):
+        """Store stuff as downloaded."""
+        self.downloaded = args
 
     def unlink(self, share_id, parent_id, node_id):
         """Store stuff as unlinked."""
         self.unlinked.append((share_id, parent_id, node_id))
 
+    def move(self, share_id, node_id, old_parent_id, new_parent_id, new_name):
+        """Store stuff as moved."""
+        data = (share_id, node_id, old_parent_id, new_parent_id, new_name)
+        self.moved.append(data)
+
 
 class BaseTestCase(testcase.BaseTwistedTestCase):
     """ Base test case """
@@ -75,8 +94,11 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
         self.shares_dir = self.mktemp('shares')
         usrdir = self.mktemp("usrdir")
         self.fsmdir = self.mktemp("fsmdir")
-        self.vm = testcase.FakeVolumeManager(usrdir)
         self.partials_dir = self.mktemp("partials")
+        # set the home for the tests
+        self.old_value = os.environ.get('HOME', None)
+        os.environ['HOME'] = self.home_dir
+        self.vm = testcase.FakeVolumeManager(usrdir)
         self.fsm = filesystem_manager.FileSystemManager(self.fsmdir,
                                                         self.partials_dir,
                                                         self.vm)
@@ -86,6 +108,10 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
         self.aq = FakeAQ()
 
     def tearDown(self):
+        if self.old_value is None:
+            os.environ.pop('HOME')
+        else:
+            os.environ['HOME'] = self.old_value
         self.rmtree(self.tmpdir)
         testcase.BaseTwistedTestCase.tearDown(self)
 
@@ -101,9 +127,11 @@ class BaseTestCase(testcase.BaseTwistedTestCase):
         fsm.vm.add_share(share)
         return share
 
-    def create_udf(self, udf_id, node_id, suggested_path, path, subscribed):
+    def create_udf(self, udf_id, node_id, suggested_path, path, subscribed,
+                   generation=None, free_bytes=100):
         """Create an UDF and add it to the volume manager."""
-        volume = volumes.UDFVolume(udf_id, node_id, suggested_path.decode('utf-8'))
+        volume = volumes.UDFVolume(udf_id, node_id, generation, free_bytes,
+                                   suggested_path.decode('utf-8'))
         udf = volume_manager.UDF.from_udf_volume(volume, path)
         udf.subscribed = subscribed
         self.fsm.vm.add_udf(udf)
@@ -204,6 +232,102 @@ class CollectionTests(BaseTestCase):
         lr.start()
         self.assertEqual(sorted(toscan), [share.path, self.vm.root.path])
 
+    def test_deleted_rw(self):
+        """Test with a deleted rw share."""
+        # create the share
+        share = self.create_share('share_id', 'rw_share',  self.fsm,
+                                  self.shares_dir, access_level='Modify')
+        self.fsm.create(share.path, "share_id", is_dir=True)
+        self.fsm.set_node_id(share.path, "uuid")
+
+        # remove the share from disk
+        shutil.rmtree(share.path)
+
+        lr = LocalRescan(self.vm, self.fsm, self.eq, self.aq)
+
+        # pylint: disable-msg=W0212
+        toscan = []
+        def f():
+            """helper"""
+            toscan.extend(x[1] for x in lr._queue)
+            return defer.Deferred()
+
+        lr._queue_scan = f
+        # patch share_deleted to check we are calling it
+        d = defer.Deferred()
+        self.patch(self.vm, 'share_deleted', d.callback)
+        lr.start()
+        vol_id = yield d
+        self.assertEqual(vol_id, share.volume_id)
+        self.assertEqual(sorted(toscan), [self.vm.root.path])
+
+    def test_deleted_rw_not_empty(self):
+        """Test with a deleted rw share with some nodes in it."""
+        # create the share
+        share = self.create_share('share_id', 'rw_share',  self.fsm,
+                                  self.shares_dir, access_level='Modify')
+        self.fsm.create(share.path, "share_id", is_dir=True)
+        self.fsm.set_node_id(share.path, "uuid")
+
+        for name in ['a', 'b', 'c']:
+            # create a node in the share
+            filepath = os.path.join(share.path, name)
+            self.fsm.create(filepath, share.id)
+            self.fsm.set_node_id(filepath, str(uuid.uuid4()))
+            open(filepath, "w").close()
+
+        # remove the share from disk
+        shutil.rmtree(share.path)
+
+        lr = LocalRescan(self.vm, self.fsm, self.eq, self.aq)
+
+        # pylint: disable-msg=W0212
+        toscan = []
+        def f():
+            """helper"""
+            toscan.extend(x[1] for x in lr._queue)
+            return defer.Deferred()
+
+        lr._queue_scan = f
+        # patch share_deleted to check we are calling it
+        d = defer.Deferred()
+        self.patch(self.vm, 'share_deleted', d.callback)
+        lr.start()
+        vol_id = yield d
+        self.assertEqual(vol_id, share.volume_id)
+        self.assertEqual(sorted(toscan), [self.vm.root.path])
+
+    @defer.inlineCallbacks
+    def test_deleted_udf(self):
+        """Test with a deleted udf."""
+        # create the udf
+        udf_path = os.path.expanduser("~/myudf")
+        udf = self.create_udf('udf_id', 'udf_root_node_id', "~/myudf",
+                                    udf_path, True)
+        os.makedirs(udf.path)
+        self.fsm.create(udf.path, 'udf_id', is_dir=True)
+
+        # remove the udf from disk
+        shutil.rmtree(udf.path)
+
+        lr = LocalRescan(self.vm, self.fsm, self.eq, self.aq)
+
+        # pylint: disable-msg=W0212
+        toscan = []
+        def f():
+            """helper"""
+            toscan.extend(x[1] for x in lr._queue)
+            return defer.Deferred()
+
+        lr._queue_scan = f
+        # patch unsubscribe_udf to check we are calling it
+        d = defer.Deferred()
+        self.patch(self.vm, 'unsubscribe_udf', d.callback)
+        lr.start()
+        vol_id = yield d
+        self.assertEqual(vol_id, udf.volume_id)
+        self.assertEqual(sorted(toscan), [self.vm.root.path])
+
 
 class VolumeTestCase(BaseTestCase):
     """Test how LocalRescan manages volumes."""
@@ -256,7 +380,7 @@ class VolumeTestCase(BaseTestCase):
         toscan = []
         def f():
             """helper"""
-            for _, path, _, _, udfmode in self.lr._queue:
+            for _, path, _, udfmode in self.lr._queue:
                 toscan.append(path)
                 self.assertFalse(udfmode)
             return defer.Deferred()
@@ -272,7 +396,7 @@ class VolumeTestCase(BaseTestCase):
         def f():
             """helper"""
             self.assertTrue(len(self.lr._queue), 1)
-            _, path, _, _, udfmode = self.lr._queue[0]
+            _, path, _, udfmode = self.lr._queue[0]
             self.assertEqual(path, udf_path)
             self.assertFalse(udfmode)
             return defer.Deferred()
@@ -287,7 +411,7 @@ class VolumeTestCase(BaseTestCase):
         def f():
             """helper"""
             self.assertTrue(len(self.lr._queue), 1)
-            _, path, _, _, udfmode = self.lr._queue[0]
+            _, path, _, udfmode = self.lr._queue[0]
             self.assertEqual(path, udf_path)
             self.assertTrue(udfmode)
             return defer.Deferred()
@@ -350,7 +474,7 @@ class VolumeTestCase(BaseTestCase):
         toscan = []
         def f():
             """helper"""
-            for _, path, _, _, udfmode in self.lr._queue:
+            for _, path, _, udfmode in self.lr._queue:
                 toscan.append(path)
             return defer.succeed(None)
 
@@ -365,7 +489,7 @@ class TwistedBase(BaseTestCase):
     timeout = 2
 
     def setUp(self):
-        """set up the test."""
+        """Set up the test."""
         BaseTestCase.setUp(self)
         self.deferred = defer.Deferred()
         self.lr = LocalRescan(self.vm, self.fsm, self.eq, self.aq)
@@ -378,6 +502,15 @@ class TwistedBase(BaseTestCase):
         self.share.node_id = "uuidshare"
         self.vm.shares['share_id'] = self.share
 
+        self.handler = testcase.MementoHandler()
+        self._logger = logging.getLogger('ubuntuone.SyncDaemon.local_rescan')
+        self._logger.addHandler(self.handler)
+
+    def tearDown(self):
+        """Tear down."""
+        BaseTestCase.tearDown(self)
+        self._logger.addHandler(self.handler)
+
     def startTest(self, check_function):
         """Start the test using lr.start()."""
         self.deferred.addCallback(lambda x: self.lr.start())
@@ -829,7 +962,7 @@ class ComparationTests(TwistedBase):
 
     def test_content_changed(self):
         """Test that it detects the content change."""
-        # create one type in the share, other in disk
+        # create a node in metadata, change it in disk
         path = self.create_node("a", is_dir=False)
         with open(path, "w") as fh:
             fh.write("foo")
@@ -837,6 +970,7 @@ class ComparationTests(TwistedBase):
         def check(_):
             """check"""
             self.assertEqual(self.eq.pushed, [('FS_FILE_CLOSE_WRITE', path)])
+            self.assertTrue(self.handler.check_debug("differ", "Old", "New"))
 
         self.startTest(check)
         return self.deferred
@@ -1011,6 +1145,7 @@ class InotifyTests(TwistedBase):
         self.deferred.callback(None)
         return self.deferred
 
+
 class QueuingTests(BaseTestCase):
     """Test that simultaneus calls are queued."""
     timeout = 2
@@ -1450,12 +1585,19 @@ class BadStateTests(TwistedBase):
 
         # start a put file, and assume we got interrupted (no server hash)
         pathhash = self._hash(path)
-        self.fsm.set_by_mdid(mdid, local_hash=pathhash)
+        self.fsm.set_by_mdid(mdid, local_hash=pathhash, crc32='foo')
         self.fsm.refresh_stat(path)
 
         def check(_):
-            """check"""
-            self.assertEqual(self.eq.pushed, [('FS_FILE_CLOSE_WRITE', path)])
+            """Compare predictable args, and check fh factory."""
+            mdobj = self.fsm.get_by_mdid(mdid)
+            self.assertEqual(self.aq.uploaded[:6],
+                             (mdobj.share_id, mdobj.node_id, mdobj.server_hash,
+                              mdobj.local_hash, mdobj.crc32, mdobj.size))
+            factory = self.aq.uploaded[-1]
+            self.assertTrue(isinstance(factory(), file))
+            self.assertTrue(self.handler.check_info("Resuming upload",
+                                                    "interrupted"))
 
         self.startTest(check)
         return self.deferred
@@ -1478,11 +1620,13 @@ class BadStateTests(TwistedBase):
         self.assertTrue(os.path.exists(partial_path))
 
         def check(_):
-            """arrange the metadata so later server_rescan will do ok"""
+            """Compare predictable args, and check fh factory."""
             mdobj = self.fsm.get_by_mdid(mdid)
-            self.assertFalse(mdobj.info.is_partial)
-            self.assertEqual(mdobj.server_hash, mdobj.local_hash)
-            self.assertFalse(os.path.exists(partial_path))
+            self.assertEqual(self.aq.downloaded[:3], (
+                             mdobj.share_id, mdobj.node_id, mdobj.server_hash))
+            factory = self.aq.downloaded[-1]
+            self.assertTrue(isinstance(factory(), file))
+            self.assertTrue(self.handler.check_debug("comp yield", "SERVER"))
 
         self.startTest(check)
         return self.deferred
@@ -1503,11 +1647,13 @@ class BadStateTests(TwistedBase):
         os.remove(partial_path)
 
         def check(_):
-            """arrange the metadata so later server_rescan will do ok"""
-            self.assertEqual(self.eq.pushed, [])
+            """Compare predictable args, and check fh factory."""
             mdobj = self.fsm.get_by_mdid(mdid)
-            self.assertFalse(mdobj.info.is_partial)
-            self.assertEqual(mdobj.server_hash, mdobj.local_hash)
+            self.assertEqual(self.aq.downloaded[:3], (
+                             mdobj.share_id, mdobj.node_id, mdobj.server_hash))
+            factory = self.aq.downloaded[-1]
+            self.assertTrue(isinstance(factory(), file))
+            self.assertTrue(self.handler.check_debug("comp yield", "SERVER"))
 
         self.startTest(check)
         return self.deferred
@@ -1516,6 +1662,8 @@ class BadStateTests(TwistedBase):
         """We were downloading the file, but it was interrupted, and changed"""
         # create the file in metadata
         path = os.path.join(self.share.path, "a")
+        with open(path, 'w') as fh:
+            fh.write("previous content")
         mdid = self.fsm.create(path, self.share.volume_id, is_dir=False)
         partial_path = os.path.join(self.fsm.partials_dir,
                                 mdid + ".u1partial." + os.path.basename(path))
@@ -1533,19 +1681,29 @@ class BadStateTests(TwistedBase):
             fh.write("I see dead people")
 
         def check(_):
-            """arrange the metadata so later server_rescan will do ok"""
+            """The MD should be left as is, and issue the CLOSE_WRITE.
+
+            This will mimic what happens when the user touches the file
+            while downloading, with SD running. As the file changed, we don't
+            start a new download, an remove the old .partial.
+            """
             mdobj = self.fsm.get_by_mdid(mdid)
-            self.assertFalse(mdobj.info.is_partial)
-            self.assertEqual(mdobj.server_hash, mdobj.local_hash)
+            self.assertTrue(mdobj.info.is_partial)
+            self.assertEqual(mdobj.local_hash, '')
+            self.assertEqual(mdobj.server_hash, "blah-hash-blah")
             self.assertFalse(os.path.exists(partial_path))
             self.assertEqual(self.eq.pushed, [('FS_FILE_CLOSE_WRITE', path)])
+            self.assertTrue(self.handler.check_debug("differ", "Old", "New"))
 
         self.startTest(check)
         return self.deferred
 
     def test_SERVER_dir(self):
-        """We were downloading the dir, but it was interrupted."""
-        # create the file in metadata
+        """Found a dir in SERVER.
+
+        This was valid before, but no more, so we just fix and log in warning.
+        """
+        # create the dir in metadata
         path = os.path.join(self.share.path, "a")
         mdid = self.fsm.create(path, self.share.volume_id, is_dir=True)
         partial_path = os.path.join(self.fsm.partials_dir,
@@ -1560,18 +1718,24 @@ class BadStateTests(TwistedBase):
         self.assertTrue(os.path.exists(partial_path))
 
         # also put a file inside the directory, to check that LR enters in it
-        filepath = os.path.join(path, "file")
-        open(filepath, "w").close()
+        fpath = os.path.join(path, "file")
+        open(fpath, "w").close()
 
         def check(_):
-            """arrange the metadata so later server_rescan will do ok"""
+            """Several checks."""
             mdobj = self.fsm.get_by_mdid(mdid)
+
+            # it should be in NONE
             self.assertFalse(mdobj.info.is_partial)
             self.assertEqual(mdobj.server_hash, mdobj.local_hash)
             self.assertFalse(os.path.exists(partial_path))
-            self.assertEqual(self.eq.pushed, [('FS_FILE_CREATE', filepath),
-                                              ('FS_FILE_CLOSE_WRITE', filepath)
-                                             ])
+
+            # file inside dir should be found
+            self.assertEqual(self.eq.pushed, [('FS_FILE_CREATE', fpath),
+                                              ('FS_FILE_CLOSE_WRITE', fpath)])
+            # logged in warning
+            self.assertTrue(self.handler.check_warning(
+                                                "Found a directory in SERVER"))
 
         self.startTest(check)
         return self.deferred
@@ -1639,10 +1803,38 @@ class BadStateTests(TwistedBase):
         open(path, "w").close()
 
         def check(_):
-            """remove everything"""
-            self.assertFalse(os.path.exists(path))
+            """Assert LR worked ok.
+
+            The partial should be gone, the path should be there (the file!),
+            with its metadata.
+            """
+            self.assertTrue(os.path.exists(path))
             self.assertFalse(os.path.exists(partial_path), partial_path)
-            self.assertFalse(self.fsm.has_metadata(path=path))
+            self.assertTrue(self.fsm.has_metadata(path=path))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_partial_for_dir_in_NONE(self):
+        """Found a .partial of a directory whose MD said changed=NONE."""
+        # create the dir in metadata
+        path = os.path.join(self.share.path, "a")
+        mdid = self.fsm.create(path, self.share.volume_id, is_dir=True)
+        self.fsm.set_node_id(path, "uuid")
+        os.mkdir(path)
+
+        # create the partial (not through FSM to don't signal the MD)
+        partial_path = os.path.join(self.fsm.partials_dir,
+                                    mdid + ".u1partial.a")
+        fh = open(partial_path, "w")
+        fh.write("foobar")
+        fh.close()
+
+        def check(_):
+            """Only partial will be removed."""
+            self.assertTrue(os.path.exists(path))
+            self.assertTrue(self.fsm.has_metadata(path=path))
+            self.assertFalse(os.path.exists(partial_path))
 
         self.startTest(check)
         return self.deferred
@@ -1670,10 +1862,14 @@ class BadStateTests(TwistedBase):
         self.fsm.fs[mdid] = real_mdobj
 
         def check(_):
-            """remove everything"""
-            self.assertFalse(os.path.exists(path), path)
+            """Assert LR worked ok.
+
+            As it's a corrupted MD situation, the safest path is to just remove
+            the partial, and leave the file as it is.
+            """
+            self.assertTrue(os.path.exists(path), path)
             self.assertFalse(os.path.exists(partial_path), partial_path)
-            self.assertFalse(self.fsm.has_metadata(path=path))
+            self.assertTrue(self.fsm.has_metadata(path=path))
 
         self.startTest(check)
         return self.deferred
@@ -1802,52 +1998,74 @@ class BadStateTests(TwistedBase):
         self.startTest(check)
         return self.deferred
 
-    def test_remove_only_partial_of_childs(self):
-        """We were downloading the file, but it was interrupted.
-        and we have a partial of another fiel in a subdir."""
+    def test_SERVER_file_ro_share(self):
+        """We were downloading the file, but it was interrupted in RO share."""
         # create the file in metadata
-        path = os.path.join(self.share.path, "a")
-        mdid = self.fsm.create(path, self.share.volume_id, is_dir=False)
+        ro_share = self.create_share('share_ro_id', 'share_ro2',  self.fsm,
+                                       self.shares_dir, access_level='View')
+        self.fsm.create(ro_share.path, ro_share.id, is_dir=True)
+        self.fsm.set_node_id(ro_share.path, "uuidshare")
+        path = os.path.join(ro_share.path, "a")
+        with self.fsm._enable_share_write(ro_share.id, path):
+            open(path, "w").close()
+        mdid = self.fsm.create(path, ro_share.volume_id, is_dir=False)
         partial_path = os.path.join(self.fsm.partials_dir,
                                 mdid + ".u1partial." + os.path.basename(path))
         self.fsm.set_node_id(path, "uuid")
 
         # start the download, never complete it
         self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
-        self.fsm.create_partial("uuid", self.share.volume_id)
-        fh = self.fsm.get_partial_for_writing("uuid", self.share.volume_id)
+        self.fsm.create_partial("uuid", ro_share.volume_id)
+        fh = self.fsm.get_partial_for_writing("uuid", ro_share.volume_id)
         fh.write("foobar")
         self.assertTrue(os.path.exists(partial_path))
 
-        # create another file with a partial, but one level deeper
-        dir = os.path.join(self.share.path, "dir")
-        os.mkdir(dir)
-        path_b = os.path.join(self.share.path, "dir", "b")
-        open(path_b, "w").close()
-        self.fsm.create(dir, self.share.volume_id, is_dir=True)
-        self.fsm.set_node_id(dir, "uuid2")
+        def check(_):
+            """Compare predictable args, and check fh factory."""
+            mdobj = self.fsm.get_by_mdid(mdid)
+            self.assertEqual(self.aq.downloaded[:3], (
+                             mdobj.share_id, mdobj.node_id, mdobj.server_hash))
+            factory = self.aq.downloaded[-1]
+            self.assertTrue(isinstance(factory(), file))
+            self.assertTrue(self.handler.check_debug("comp yield", "SERVER"))
 
-        mdid_b = self.fsm.create(path_b, self.share.volume_id, is_dir=False)
-        partial_path_b = os.path.join(self.fsm.partials_dir,
-                                mdid_b + ".u1partial." + os.path.basename(path_b))
-        self.fsm.set_node_id(path_b, "uuid1")
+        self.startTest(check)
+        return self.deferred
 
+    def test_SERVER_dir_ro_share(self):
+        """Found a dir in SERVER in a ro_share.
+
+        This was valid before, but no more, so we just fix and log in warning.
+        """
+        # create the dir in metadata
+        ro_share = self.create_share('share_ro_id', 'share_ro2',  self.fsm,
+                                       self.shares_dir, access_level='View')
+        self.fsm.create(ro_share.path, ro_share.id, is_dir=True)
+        self.fsm.set_node_id(ro_share.path, "uuidshare")
+
+        path = os.path.join(ro_share.path, "a")
+        mdid = self.fsm.create(path, ro_share.volume_id, is_dir=True)
+        partial_path = os.path.join(self.fsm.partials_dir,
+                                mdid + ".u1partial." + os.path.basename(path))
+        self.fsm.set_node_id(path, "uuid")
 
         # start the download, never complete it
-        self.fsm.set_by_mdid(mdid_b, server_hash="blah-hash-blah-b")
-        self.fsm.create_partial("uuid1", self.share.volume_id)
-        fh = self.fsm.get_partial_for_writing("uuid1", self.share.volume_id)
+        self.fsm.set_by_mdid(mdid, server_hash="blah-hash-blah")
+        self.fsm.create_partial("uuid", ro_share.volume_id)
+        fh = self.fsm.get_partial_for_writing("uuid", ro_share.volume_id)
         fh.write("foobar")
-        self.assertTrue(os.path.exists(partial_path_b))
+        self.assertTrue(os.path.exists(partial_path))
 
         def check(_):
-            """arrange the metadata so later server_rescan will do ok"""
+            """Several checks."""
             mdobj = self.fsm.get_by_mdid(mdid)
+            # it should be in NONE
             self.assertFalse(mdobj.info.is_partial)
             self.assertEqual(mdobj.server_hash, mdobj.local_hash)
             self.assertFalse(os.path.exists(partial_path))
-            self.assertFalse(os.path.exists(partial_path_b))
-            self.assertTrue(os.path.exists(path_b))
+            # logged in warning
+            self.assertTrue(self.handler.check_warning(
+                                                "Found a directory in SERVER"))
 
         self.startTest(check)
         return self.deferred
@@ -1923,19 +2141,20 @@ class RootBadStateTests(TwistedBase):
         return self._test_it(udf)
 
 
-class TrashTests(TwistedBase):
-    """Test handling trash."""
+class LimboTests(TwistedBase):
+    """Test handling limbos."""
 
     def test_nothing(self):
-        """No trash."""
+        """No trash, no moves."""
         def check(_):
-            """check"""
+            """Check."""
             self.assertEqual(self.aq.unlinked, [])
+            self.assertEqual(self.aq.moved, [])
 
         self.startTest(check)
         return self.deferred
 
-    def test_one(self):
+    def test_trash_one(self):
         """Something in the trash."""
         path = os.path.join(self.share.path, "a")
         mdid = self.fsm.create(path, self.share.volume_id, is_dir=True)
@@ -1943,14 +2162,17 @@ class TrashTests(TwistedBase):
         self.fsm.delete_to_trash(mdid, "parent_id")
 
         def check(_):
-            """check"""
+            """Check."""
+            self.assertEqual(self.aq.moved, [])
             self.assertEqual(self.aq.unlinked,
                              [(self.share.volume_id, "parent_id", "uuid")])
+            self.assertTrue(self.handler.check_info(
+                            "generating Unlink from trash"))
 
         self.startTest(check)
         return self.deferred
 
-    def test_two(self):
+    def test_trash_two(self):
         """Two nodes (file and dir) in the trash."""
         path = os.path.join(self.share.path, "a")
         mdid1 = self.fsm.create(path, self.share.volume_id, is_dir=True)
@@ -1963,7 +2185,8 @@ class TrashTests(TwistedBase):
         self.fsm.delete_to_trash(mdid2, "parent_id")
 
         def check(_):
-            """check"""
+            """Check."""
+            self.assertEqual(self.aq.moved, [])
             self.assertEqual(sorted(self.aq.unlinked),
                              [(self.share.volume_id, "parent_id", "uuid1"),
                               (self.share.volume_id, "parent_id", "uuid2")])
@@ -1971,6 +2194,126 @@ class TrashTests(TwistedBase):
         self.startTest(check)
         return self.deferred
 
+    def test_no_double_unlink(self):
+        """Avoid double unlinks.
+
+        This happens because we signal a deletion and move to trash, and then
+        delete from trash.
+
+        There's no way to test this here, because it will need all working
+        machinery (EQ, Sync, FSM) and not mockups, so we just look into the
+        logs to assert order.
+        """
+        def check(_):
+            """Check"""
+            msgs = [x.msg for x in self.handler.records]
+            pos_trash = msgs.index('processing trash')
+            pos_compares = msgs.index('comparing directory %r')
+            self.assertTrue(pos_trash < pos_compares)
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_trash_node_marker(self):
+        """Trash with node_id being a marker."""
+        path = os.path.join(self.share.path, "a")
+        mdid = self.fsm.create(path, self.share.volume_id, is_dir=True)
+        self.fsm.delete_to_trash(mdid, "parent_id")
+
+        def check(_):
+            """Check."""
+            self.assertEqual(self.aq.unlinked, [])
+            self.assertTrue(self.handler.check_info("removing from trash"))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_trash_parent_marker(self):
+        """Trash with parent_id being a marker."""
+        path = os.path.join(self.share.path, "a")
+        mdid = self.fsm.create(path, self.share.volume_id, is_dir=True)
+        self.fsm.set_node_id(path, "uuid")
+        self.fsm.delete_to_trash(mdid, MDMarker("parent_id"))
+
+        def check(_):
+            """Check."""
+            self.assertEqual(self.aq.unlinked, [])
+            self.assertTrue(self.handler.check_info("removing from trash"))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_move_limbo_one(self):
+        """Something in the move_limbo."""
+        self.fsm.add_to_move_limbo("share", "uuid", "old_parent",
+                                   "new_parent", "new_name")
+
+        def check(_):
+            """Check."""
+            self.assertEqual(self.aq.unlinked, [])
+            self.assertEqual(self.aq.moved, [("share", "uuid", "old_parent",
+                                             "new_parent", "new_name")])
+            self.assertTrue(self.handler.check_info(
+                            "generating Move from limbo"))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_move_limbo_two(self):
+        """Two nodes (file and dir) in the move_limbo."""
+        n1 = ("share1", "uuid1", "old_parent1", "new_parent1", "new_name1")
+        n2 = ("share2", "uuid2", "old_parent2", "new_parent2", "new_name2")
+        self.fsm.add_to_move_limbo(*n1)
+        self.fsm.add_to_move_limbo(*n2)
+
+        def check(_):
+            """Check."""
+            self.assertEqual(self.aq.unlinked, [])
+            self.assertEqual(sorted(self.aq.moved), [n1, n2])
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_mixed_trash_moves(self):
+        """Mixed limbo with trash and moves."""
+        path = os.path.join(self.share.path, "a")
+        mdid = self.fsm.create(path, self.share.volume_id, is_dir=True)
+        self.fsm.set_node_id(path, "uuid")
+        self.fsm.delete_to_trash(mdid, "parent_id")
+
+        self.fsm.add_to_move_limbo("share", "uuid", "old_parent",
+                                   "new_parent", "new_name")
+
+        def check(_):
+            """Check."""
+            self.assertEqual(self.aq.moved, [("share", "uuid", "old_parent",
+                                             "new_parent", "new_name")])
+            self.assertEqual(self.aq.unlinked,
+                             [(self.share.volume_id, "parent_id", "uuid")])
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_move_limbo_markers(self):
+        """Trash with node_id in None."""
+        # load 4 things in move limbo, each with a MDMarker (not for 'name')
+        n = ("share_id", "node_id", "old_parent_id", "new_parent_id", "name")
+        for i in range(4):
+            m = list(n)
+            m[1] = 'node_id_%d' % i
+            m[i] = MDMarker(m[i])
+        self.fsm.add_to_move_limbo(*m)
+
+        def check(_):
+            """Check."""
+            self.assertEqual(self.aq.moved, [])
+            self.assertTrue(self.handler.check_info(
+                            "removing from move limbo"))
+            self.assertFalse(self.handler.check_info("generating Move"))
+
+        self.startTest(check)
+        return self.deferred
+
 
 class ParentWatchForUDFTestCase(BaseTestCase):
     """Tests over watches for UDF's parent dir."""
@@ -2061,6 +2404,53 @@ class ParentWatchForUDFTestCase(BaseTestCase):
         return self._deferred
 
 
+class BrokenNodesTests(TwistedBase):
+    """Test that LR logs all broken nodes at start."""
+
+    def test_nothing(self):
+        """No broken nodes."""
+        def check(_):
+            """Check"""
+            self.assertFalse(self.handler.check_info('Broken node'))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_one(self):
+        """Something in the broken nodes list."""
+        path = os.path.join(self.share.path, "brokennodepath")
+        mdid = self.fsm.create(path, self.share.volume_id, node_id="uuid")
+        self.fsm.set_by_mdid(mdid, dirty=True, local_hash='foo')
+
+        def check(_):
+            """Check"""
+            self.assertTrue(self.handler.check_info('Broken node',
+                                                    'brokennodepath', mdid))
+
+        self.startTest(check)
+        return self.deferred
+
+    def test_several(self):
+        """Several in the broken nodes list."""
+        path1 = os.path.join(self.share.path, "brokenpath1")
+        mdid1 = self.fsm.create(path1, self.share.volume_id, node_id='uuid1')
+        self.fsm.set_by_mdid(mdid1, dirty=True, local_hash='foo')
+        path2 = os.path.join(self.share.path, "brokenpath2")
+        mdid2 = self.fsm.create(path2, self.share.volume_id, node_id='uuid2')
+        self.fsm.set_by_mdid(mdid2, dirty=True, local_hash='foo')
+
+        def check(_):
+            """Check"""
+            self.assertTrue(self.handler.check_info('Broken node',
+                                                    'brokenpath1', mdid1))
+
+            self.assertTrue(self.handler.check_info('Broken node',
+                                                    'brokenpath2', mdid2))
+
+        self.startTest(check)
+        return self.deferred
+
+
 def test_suite():
     # pylint: disable-msg=C0111
     return unittest.TestLoader().loadTestsFromName(__name__)
