@@ -19,13 +19,23 @@
 
 import new
 import os
-import gnomekeyring
 
-from contrib.mocker import MockerTestCase
-from contrib.testing.testcase import DBusTwistedTestCase, FakeLogin
+import dbus.service
+import logging
+import ubuntu_sso
+
+from mocker import MockerTestCase, ANY
 from twisted.internet import defer
 from twisted.python.failure import Failure
-from ubuntuone.syncdaemon import dbus_interface, tools
+from twisted.trial.unittest import TestCase
+
+from contrib.testing.testcase import (
+    DBusTwistedTestCase,
+    FAKED_CREDENTIALS,
+    FakeLogin,
+    MementoHandler)
+from ubuntuone import clientdefs
+from ubuntuone.syncdaemon import tools
 
 class SyncDaemonTool(tools.SyncDaemonTool):
     """A subclass of tools.SyncDaemonTool that keep track of the deferreds."""
@@ -59,39 +69,14 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
     _path = os.path.join(os.getcwd(), "bin", "ubuntuone-preferences")
     u1prefs = new.module('u1prefs')
     execfile(_path, u1prefs.__dict__)
-    u1prefs.DBUS_IFACE_AUTH_PATH = '/oauthdesktop'
+    u1prefs.DBUS_CRED_PATH = '/oauthdesktop'
 
     def setUp(self):
         MockerTestCase.setUp(self)
         DBusTwistedTestCase.setUp(self)
         self.oauth = FakeLogin(self.bus)
-        self._old_path = dbus_interface.DBUS_PATH_AUTH
-        dbus_interface.DBUS_PATH_AUTH = '/oauthdesktop'
-
-        # For testing keyring queries
-        self.keyring = self.mocker.mock()
-        self.item = self.mocker.mock(gnomekeyring.Found)
-
-        self.item_id = 999
-
-        self.item.item_id
-        self.mocker.result(self.item_id)
-        self.mocker.count(0, None)
-
-        self.item.secret
-        self.mocker.result('oauth_token=access_key'
-                           '&oauth_token_secret=access_secret')
-        self.mocker.count(0, None)
-
-        self.keyring.find_items_sync(
-            None,
-            {'ubuntuone-realm': 'https://ubuntuone.com',
-             'oauth-consumer-key': 'ubuntuone'})
-        self.mocker.count(0, None)
-        self.mocker.result([self.item])
-        self.keyring.ITEM_GENERIC_SECRET
-        self.mocker.count(0, None)
-        self.mocker.result(None)
+        self._old_path = ubuntu_sso.DBUS_CRED_PATH
+        ubuntu_sso.DBUS_CRED_PATH = '/oauthdesktop'
 
         self.u1prefs.make_rest_request = self.make_rest_request
         self.u1prefs.SyncDaemonTool = SyncDaemonTool
@@ -107,11 +92,11 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
                         signal_receivers.update(match)
         yield self.cleanup_signal_receivers(signal_receivers)
         self.oauth.shutdown()
-        dbus_interface.DBUS_PATH_AUTH = self._old_path
+        ubuntu_sso.DBUS_AUTH = self._old_path
         yield DBusTwistedTestCase.tearDown(self)
 
     def make_rest_request(self, url=None, method='GET',
-                          callback=None, keyring=None):
+                          callback=None):
         """Override the real request call to mock some stuff."""
         if callback:
             callback(self.content)
@@ -121,7 +106,8 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
     def test_bw_throttling(self):
         """Test that toggling bw throttling works correctly."""
         self.mocker.replay()
-        widget = self.u1prefs.DevicesWidget(None, keyring=self.keyring)
+        widget = self.u1prefs.DevicesWidget(None)
+        self.assertFalse(widget.got_limits)
         widget.update_bw_settings = self.mocker.mock()
         try:
             widget.devices = []
@@ -141,12 +127,22 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
             self.assertTrue(widget.up_spin.get_property('sensitive') and
                             widget.dn_spin.get_property('sensitive'),
                             "the spinbuttons should now be sensitive")
+
+            self.assertFalse(widget.got_limits,
+                             "got_limits should be False before getting the limits from SD")
+            widget.handle_limits({ 'upload': 1097152, 'download': 1097152 })
+            self.assertEqual(widget.up_limit, 1097152,
+                             "upload_limit should be 1097152")
+            self.assertEqual(widget.dn_limit, 1097152,
+                             "download_limit should be 1097152")
+            self.assertTrue(widget.got_limits,
+                            "got_limits should be True after getting the limits from SD")
         finally:
             widget.destroy()
 
     def test_list_devices_fills_devices_list_with_fake_result_when_empty(self):
         self.mocker.replay()
-        widget = self.u1prefs.DevicesWidget(None, keyring=self.keyring)
+        widget = self.u1prefs.DevicesWidget(None)
         widget.update_bw_settings = self.mocker.mock()
         try:
             widget.devices = []
@@ -160,7 +156,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
 
     def test_list_devices_shows_devices_list(self):
         self.mocker.replay()
-        widget = self.u1prefs.DevicesWidget(None, keyring=self.keyring)
+        widget = self.u1prefs.DevicesWidget(None)
         widget.update_bw_settings = self.mocker.mock()
         try:
             widget.devices = []
@@ -178,7 +174,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
             # check a placeholder for the local machine description is there
             self.assertTrue(('Label', '<LOCAL MACHINE>') in interesting)
             # check the bw limitation stuff is there
-            self.assertTrue(('CheckButton', '_Limit Bandwidth Usage')
+            self.assertTrue(('CheckButton', '_Limit bandwidth')
                             in interesting)
             self.assertTrue(('Label', 'Maximum _download speed (KB/s):')
                             in interesting)
@@ -191,7 +187,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
 
     def test_list_devices_shows_real_devices_list(self):
         self.mocker.replay()
-        widget = self.u1prefs.DevicesWidget(None, keyring=self.keyring)
+        widget = self.u1prefs.DevicesWidget(None)
         widget.update_bw_settings = self.mocker.mock()
         try:
             widget.devices = [{'kind': 'Computer',
@@ -228,7 +224,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
     def test_quota_display(self):
         """Test that quota display works correctly."""
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         self.assertEqual(dialog.usage_graph.get_fraction(), 0.0)
         dialog.update_quota_display(1024, 2048)
@@ -240,7 +236,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         """Test that we can request the quota info properly."""
         self.content = {"total":2048, "used":1024}
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         self.assertEqual(dialog.usage_graph.get_fraction(), 0.0)
         dialog.request_quota_info()
@@ -252,7 +248,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         """Test that the quota notice is not visible if usage is low."""
         self.content = {"total":2048, "used":1024}
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.request_quota_info()
         # the label should just be the blank '\n'
@@ -269,12 +265,12 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         """
         self.content = {"total": 100, "used": 95}
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.request_quota_info()
         # don't check the exact text, as it will probably
         # change. Check we're setting some amount of text,
-        self.assertTrue(len(dialog.overquota_label.get_text()) > 100)
+        self.assertTrue(len(dialog.overquota_label.get_text()) > 50)
         # and check that we mention upgrading
         self.assertTrue('upgrad' in dialog.overquota_label.get_text().lower())
         # and check that the icon used is just informational
@@ -283,21 +279,21 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         dialog.destroy()
         return defer.DeferredList(dialog.sdtool._deferreds)
 
-    def test_overquota_notice_and_not_upgrade_offer_on_paid_high_usage(self):
+    def test_overquota_notice_and_upgrade_offer_on_paid_high_usage(self):
         """Test that the quota notice is visible if usage is 95%.
 
         Also make sure the label does not mention upgrading.
         """
         self.content = {"total": 50<<30, "used": 49<<30}
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.request_quota_info()
         # don't check the exact text, as it will probably
         # change. Check we're setting some amount of text,
         self.assertTrue(len(dialog.overquota_label.get_text()) > 100)
         # and check that we mention upgrading
-        self.assertTrue('upgrad' not in
+        self.assertTrue('upgrad' in
                         dialog.overquota_label.get_text().lower())
         # and check that the icon used is just informational
         self.assertEqual(dialog.overquota_img.get_icon_name()[0],
@@ -312,7 +308,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         """
         self.content = {"total": 100, "used": 100}
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.request_quota_info()
         # don't check the exact text, as it will probably
@@ -330,7 +326,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         """Test that the quota notice is visible if usage is 100%."""
         self.content = {"total": 50<<30, "used": 50<<30}
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.request_quota_info()
         # don't check the exact text, as it will probably
@@ -345,24 +341,30 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
     def test_request_account_info(self):
         """Test that we can request the account info properly."""
         self.content = {"username": "ubuntuone", "nickname": "Ubuntu One",
-                        "email": "uone@example.com"}
+                        "email": "uone@example.com",
+                        "current_plan": "bananas"}
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.request_account_info()
-        self.content = {"total":2048, "used":1024}
-        dialog.request_quota_info()
         self.assertEqual(dialog.name_label.get_text(), 'Ubuntu One')
         self.assertEqual(dialog.mail_label.get_text(), 'uone@example.com')
-        # ensure the plan label says "Free" and the upgrade button is there
-        self.assertEqual(dialog.plan_label.get_text(), 'Free')
-        self.assertTrue(dialog.upgrade_link.get_visible())
-        # whoops, the user upgraded
-        self.content = {"total": 50<<30, "used":1024}
-        dialog.request_quota_info()
-        # ensure the plan label says "Paid" and the upgrade button is not there
-        self.assertEqual(dialog.plan_label.get_text(), 'Paid')
-        self.assertFalse(dialog.upgrade_link.get_visible())
+        self.assertEqual(dialog.plan_label.get_text(), 'bananas')
+        dialog.destroy()
+        return defer.DeferredList(dialog.sdtool._deferreds)
+
+    def test_request_account_info_old(self):
+        """check the old way works still (until M, server replies this way)"""
+        self.content = {"username": "ubuntuone", "nickname": "Ubuntu One",
+                        "email": "uone@example.com",
+                        "subscription": {"description": "kiwi"}}
+        self.mocker.replay()
+        dialog = self.u1prefs.UbuntuOneDialog()
+        self.assertTrue(dialog is not None)
+        dialog.request_account_info()
+        self.assertEqual(dialog.name_label.get_text(), 'Ubuntu One')
+        self.assertEqual(dialog.mail_label.get_text(), 'uone@example.com')
+        self.assertEqual(dialog.plan_label.get_text(), 'kiwi')
         dialog.destroy()
         return defer.DeferredList(dialog.sdtool._deferreds)
 
@@ -373,7 +375,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         self.expect(toggle_db_sync('bookmarks', True))
         self.expect(toggle_db_sync('bookmarks', False))
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.toggle_db_sync = toggle_db_sync
         dialog.bookmarks_check.set_active(True)
@@ -392,7 +394,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         self.expect(toggle_db_sync('contacts', True))
         self.expect(toggle_db_sync('contacts', False))
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.toggle_db_sync = toggle_db_sync
         dialog.abook_check.set_active(True)
@@ -408,7 +410,7 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
     def test_toggle_files(self):
         """Test toggling the files service on/off."""
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         dialog.files_check.set_active(True)
         self.assertTrue(dialog.files_check.get_active())
@@ -418,11 +420,50 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
         self.assertTrue(dialog.files_check.get_active())
         dialog.destroy()
         return defer.DeferredList(dialog.sdtool._deferreds)
+    
+    def test_music_metadata_not_present(self):
+        """Test dialog when the music folder does not exist."""
+        # use mocker to replace the os.path.exists func so that
+        # we can set the expectations
+        os_path_exists = self.mocker.replace('os.path.exists')
+        os_path_exists(self.u1prefs.U1MSPATH)
+        self.mocker.result(False)
+        dialog = self.u1prefs.UbuntuOneDialog()
+        # mock the sync daemon tool to ensure that the method
+        # is not called
+        sdtool = self.mocker.mock()
+        sdtool.is_files_sync_enabled()
+        self.mocker.result(True)
+        dialog.sdtool = sdtool
+        self.mocker.replay()
+        dialog.get_syncdaemon_sync_config()
+
+    def test_music_metadata_present(self):
+        """Test dialog when the music folder does exist."""
+        # use mocker to replace oos.path.exist
+        os_path_exists = self.mocker.replace('os.path.exists')
+        os_path_exists(self.u1prefs.U1MSPATH)
+        self.mocker.result(True)
+        dialog = self.u1prefs.UbuntuOneDialog()
+        # mock the sync daemon tool to ensure that the method
+        # is called as expected
+        sdtool = self.mocker.mock()
+        # create mock that will be used to add the callbacks
+        deferred = self.mocker.mock()
+        sdtool.get_folder_info(self.u1prefs.U1MSPATH)
+        self.mocker.result(deferred)
+        # use special param ANY, we really do not care about the callbacks
+        deferred.addCallbacks(ANY, ANY)
+        sdtool.is_files_sync_enabled()
+        self.mocker.result(True)
+        dialog.sdtool = sdtool
+        self.mocker.replay()
+        dialog.get_syncdaemon_sync_config()
 
     def test_toggle_files_and_music(self):
         """Test toggling the files and music services on/off."""
         self.mocker.replay()
-        dialog = self.u1prefs.UbuntuOneDialog(keyring=self.keyring)
+        dialog = self.u1prefs.UbuntuOneDialog()
         self.assertTrue(dialog is not None)
         def files_toggled(checkbutton):
             enabled = checkbutton.get_active()
@@ -487,3 +528,44 @@ class PreferencesTests(MockerTestCase, DBusTwistedTestCase):
                                       error_handler=got_dbus_error)
 
         return d
+    test_login_check.skip = \
+        "This shouldn't depend on the real DBus service (see bug #591340)."
+
+
+class CallbacksTest(TestCase, MockerTestCase):
+    """Tests for DBus callbacks."""
+
+    _path = os.path.join(os.getcwd(), "bin", "ubuntuone-preferences")
+    u1prefs = new.module('u1prefs')
+    execfile(_path, u1prefs.__dict__)
+    u1prefs.DBUS_CRED_PATH = '/oauthdesktop'
+
+    def setUp(self):
+        """Set up the test."""
+        TestCase.setUp(self)
+        self.memento = MementoHandler()
+        logger = logging.getLogger("ubuntuone-preferences")
+        logger.addHandler(self.memento)
+
+        # Create fake UbuntuoneLoginHandler object
+        self.patch(dbus, "SessionBus", lambda *a, **k: self.mocker.mock())
+        self.patch(dbus.service, "BusName", lambda *a, **k: None)
+        self.patch(dbus.service, "Object", lambda *a, **k: None)
+        self.patch(self.u1prefs.UbuntuoneLoginHandler, "present", lambda *a, **k: None)
+
+        self.login_handler = self.u1prefs.UbuntuoneLoginHandler(dialog=None)
+
+    def test_got_newcredentials(self):
+        """Test CredentialsFound signal."""
+        self.login_handler.got_newcredentials(clientdefs.APP_NAME, FAKED_CREDENTIALS)
+        self.assertTrue(self.memento.check_info("credentials", clientdefs.APP_NAME))
+
+    def test_got_credentialserror(self):
+        """Test CredentialsError signal."""
+        self.login_handler.got_credentialserror(clientdefs.APP_NAME, "Error", "Detailed error")
+        self.assertTrue(self.memento.check_error(clientdefs.APP_NAME, "Error", "Detailed error"))
+
+    def test_got_authdenied(self):
+        """Test AuthorizationDenied signal."""
+        self.login_handler.got_authdenied(clientdefs.APP_NAME)
+        self.assertTrue(self.memento.check_error(clientdefs.APP_NAME))
