@@ -43,28 +43,34 @@ EVENTS = {
     'FS_DIR_MOVE': ('path_from', 'path_to',),
     'FS_INVALID_NAME': ('dirname', 'filename',),
 
-    'AQ_FILE_NEW_OK': ('marker', 'new_id'),
-    'AQ_FILE_NEW_ERROR': ('marker', 'error'),
-    'AQ_DIR_NEW_OK': ('marker', 'new_id'),
-    'AQ_DIR_NEW_ERROR': ('marker', 'error'),
-    'AQ_MOVE_OK': ('share_id', 'node_id'),
+    'AQ_FILE_NEW_OK': ('volume_id', 'marker', 'new_id', 'new_generation'),
+    'AQ_FILE_NEW_ERROR': ('marker', 'failure'),
+    'AQ_DIR_NEW_OK': ('volume_id', 'marker', 'new_id', 'new_generation'),
+    'AQ_DIR_NEW_ERROR': ('marker', 'failure'),
+    'AQ_MOVE_OK': ('share_id', 'node_id', 'new_generation'),
     'AQ_MOVE_ERROR': ('share_id', 'node_id',
                       'old_parent_id', 'new_parent_id', 'new_name', 'error'),
-    'AQ_UNLINK_OK': ('share_id', 'parent_id', 'node_id'),
+    'AQ_UNLINK_OK': ('share_id', 'parent_id', 'node_id', 'new_generation'),
     'AQ_UNLINK_ERROR': ('share_id', 'parent_id', 'node_id', 'error'),
     'AQ_DOWNLOAD_STARTED': ('share_id', 'node_id', 'server_hash'),
+    'AQ_DOWNLOAD_FILE_PROGRESS': ('share_id', 'node_id',
+                                  'n_bytes_read', 'deflated_size'),
     'AQ_DOWNLOAD_COMMIT': ('share_id', 'node_id', 'server_hash'),
     'AQ_DOWNLOAD_FINISHED': ('share_id', 'node_id', 'server_hash'),
     'AQ_DOWNLOAD_ERROR': ('share_id', 'node_id', 'server_hash', 'error'),
     'AQ_DOWNLOAD_CANCELLED': ('share_id', 'node_id', 'server_hash'),
     'AQ_DOWNLOAD_DOES_NOT_EXIST': ('share_id', 'node_id'),
     'AQ_UPLOAD_STARTED' : ('share_id', 'node_id', 'hash'),
-    'AQ_UPLOAD_FINISHED': ('share_id', 'node_id', 'hash'),
+    'AQ_UPLOAD_FILE_PROGRESS': ('share_id', 'node_id',
+                                'n_bytes_written', 'deflated_size'),
+    'AQ_UPLOAD_FINISHED': ('share_id', 'node_id', 'hash', 'new_generation'),
     'AQ_UPLOAD_ERROR': ('share_id', 'node_id', 'error', 'hash'),
     'AQ_SHARES_LIST': ('shares_list',),
     'AQ_LIST_SHARES_ERROR': ('error',),
     'AQ_CREATE_SHARE_OK': ('share_id', 'marker'),
     'AQ_CREATE_SHARE_ERROR': ('marker', 'error'),
+    'AQ_DELETE_SHARE_OK': ('share_id',),
+    'AQ_DELETE_SHARE_ERROR': ('share_id', 'error'),
     'AQ_QUERY_ERROR': ('item', 'error'),
     'AQ_ANSWER_SHARE_OK': ('share_id', 'answer'),
     'AQ_ANSWER_SHARE_ERROR': ('share_id', 'answer', 'error'),
@@ -81,20 +87,22 @@ EVENTS = {
     'AQ_CHANGE_PUBLIC_ACCESS_ERROR': ('share_id', 'node_id', 'error'),
     'AQ_PUBLIC_FILES_LIST_OK': ('public_files',),
     'AQ_PUBLIC_FILES_LIST_ERROR': ('error',),
+    'AQ_DELTA_OK': ('volume_id', 'delta_content', 'end_generation',
+                    'full', 'free_bytes'),
+    'AQ_DELTA_ERROR': ('volume_id', 'error'),
+    'AQ_DELTA_NOT_POSSIBLE': ('volume_id',),
+    'AQ_RESCAN_FROM_SCRATCH_OK': ('volume_id', 'delta_content', 'end_generation',
+                    'free_bytes'), # must always be full
+    'AQ_RESCAN_FROM_SCRATCH_ERROR': ('volume_id', 'error'),
 
     'SV_SHARE_CHANGED': ('info',),
     'SV_SHARE_DELETED': ('share_id',),
     'SV_SHARE_ANSWERED': ('share_id', 'answer'),
-    'SV_HASH_NEW': ('share_id', 'node_id', 'hash'),
-    'SV_FILE_NEW': ('share_id', 'node_id', 'parent_id', 'name'),
-    'SV_DIR_NEW': ('share_id', 'node_id', 'parent_id', 'name'),
-    'SV_FILE_DELETED': ('share_id', 'node_id'),
-    'SV_MOVED': ('share_id', 'node_id', 'new_share_id', 'new_parent_id',
-                 'new_name'),
     'SV_FREE_SPACE': ('share_id', 'free_bytes'),
     'SV_ACCOUNT_CHANGED': ('account_info',),
     'SV_VOLUME_CREATED': ('volume',),
     'SV_VOLUME_DELETED': ('volume_id',),
+    'SV_VOLUME_NEW_GENERATION': ('volume_id', 'generation'),
 
     'HQ_HASH_NEW': ('path', 'hash', 'crc32', 'size', 'stat'),
     'HQ_HASH_ERROR': ('mdid',),
@@ -119,17 +127,18 @@ EVENTS = {
     'SYS_AUTH_ERROR': ('error',),
     'SYS_AUTH_OK': (),
     'SYS_SERVER_RESCAN_DONE': (),
+    'SYS_SERVER_RESCAN_ERROR': ('error',),
     'SYS_META_QUEUE_WAITING': (),
-    'SYS_META_QUEUE_BLOCKED': (),
     'SYS_META_QUEUE_DONE': (),
     'SYS_CONTENT_QUEUE_WAITING': (),
-    'SYS_CONTENT_QUEUE_BLOCKED': (),
     'SYS_CONTENT_QUEUE_DONE': (),
     'SYS_UNKNOWN_ERROR': (),
     'SYS_HANDSHAKE_TIMEOUT': (),
-    'SYS_ROOT_RECEIVED': ('root_id',),
+    'SYS_ROOT_RECEIVED': ('root_id', 'mdid'),
     'SYS_ROOT_MISMATCH': ('root_id', 'new_root_id'),
     'SYS_SERVER_ERROR': ('error',),
+    'SYS_QUOTA_EXCEEDED': ('volume_id', 'free_bytes'),
+    'SYS_BROKEN_NODE': ('volume_id', 'node_id', 'path', 'mdid'),
 
     'VM_UDF_SUBSCRIBED': ('udf',),
     'VM_UDF_SUBSCRIBE_ERROR': ('udf_id', 'error'),
@@ -138,6 +147,8 @@ EVENTS = {
     'VM_UDF_CREATED': ('udf',),
     'VM_UDF_CREATE_ERROR': ('path', 'error'),
     'VM_SHARE_CREATED': ('share_id',),
+    'VM_SHARE_DELETED': ('udf',),
+    'VM_SHARE_DELETE_ERROR': ('path', 'error'),
     'VM_VOLUME_DELETED': ('volume',),
     'VM_VOLUME_DELETE_ERROR': ('volume_id', 'error'),
     'VM_SHARE_CHANGED': ('share_id',),
@@ -471,6 +482,7 @@ class _GeneralINotifyProcessor(pyinotify.ProcessEvent):
                             if not this_is_a_dir:
                                 self.eq_push('FS_FILE_CLOSE_WRITE', t_path)
                         else:
+                            self.eq.inotify_watch_fix(f_path, t_path)
                             self.eq_push(evtname+"MOVE", f_path, t_path)
                     elif is_to_forreal:
                         # this is the case of a MOVE from something ignored
@@ -577,7 +589,11 @@ class _GeneralINotifyProcessor(pyinotify.ProcessEvent):
         return False
 
     def handle_dir_delete(self, fullpath):
-        """Handle the case of move a dir to a non-watched directory."""
+        """Some special work when a directory is deleted."""
+        # remove the watch on that dir from our structures
+        self.eq.inotify_rm_watch(fullpath)
+
+        # handle the case of move a dir to a non-watched directory
         paths = self.eq.fs.get_paths_starting_with(fullpath,
                                                    include_base=False)
         paths.sort(reverse=True)
@@ -585,6 +601,7 @@ class _GeneralINotifyProcessor(pyinotify.ProcessEvent):
             m = "Pushing deletion because of parent dir move: (is_dir=%s) %r"
             self.log.info(m, is_dir, path)
             if is_dir:
+                self.eq.inotify_rm_watch(path)
                 self.eq_push('FS_DIR_DELETE', path)
             else:
                 self.eq_push('FS_FILE_DELETE', path)
@@ -675,13 +692,12 @@ class EventQueue(object):
             w_dict = self._ancestors_watchs
             w_manager = self._inotify_ancestors_wm
         else:
-            raise ValueError("The path %r is not watched right now!" % dirpath)
+            self.log.warning("Tried to remove a nonexistent watch on %r",
+                             dirpath)
+            return
 
-        wd = w_dict[dirpath]
-        result = w_manager.rm_watch(wd)
-        if not result[wd]:
-            raise RuntimeError("The path %r couldn't be removed!" % dirpath)
-        del w_dict[dirpath]
+        wd = w_dict.pop(dirpath)
+        w_manager.rm_watch(wd)
 
     def inotify_add_watch(self, dirpath):
         """Add watch to a dir."""
@@ -707,6 +723,20 @@ class EventQueue(object):
         return (dirpath in self._general_watchs or
                 dirpath in self._ancestors_watchs)
 
+    def inotify_watch_fix(self, pathfrom, pathto):
+        """Fix the path in inotify structures."""
+        if pathfrom in self._general_watchs:
+            wdict = self._general_watchs
+        elif pathfrom in self._ancestors_watchs:
+            wdict = self._ancestors_watchs
+        else:
+            m = "Tried to fix nonexistent path %r in watches (to %r)"
+            self.log.warning(m, pathfrom, pathto)
+            return
+
+        # fix
+        wdict[pathto] = wdict.pop(pathfrom)
+
     def _is_udf_ancestor(self, path):
         """Decide if path is an UDF ancestor or not."""
         for udf in self.fs.vm.udfs.itervalues():
@@ -743,6 +773,8 @@ class EventQueue(object):
         if event_name.endswith('DELETE'):
             # log every DELETE in INFO level
             self.log.info(log_msg, event_name, args, kwargs)
+        elif event_name == 'SYS_USER_CONNECT':
+            self.log.debug(log_msg, event_name, '*', '*')
         else:
             self.log.debug(log_msg, event_name, args, kwargs)
 
