@@ -27,12 +27,14 @@ import logging
 import contextlib
 import errno
 import stat
+import uuid
 
 from ubuntuone.syncdaemon import file_shelf
 from ubuntuone.syncdaemon.volume_manager import VolumeDoesNotExist
-import uuid
+from ubuntuone.syncdaemon.interfaces import IMarker
+from ubuntuone.syncdaemon.marker import MDMarker
 
-METADATA_VERSION = "4"
+METADATA_VERSION = "5"
 
 #
 # File System Manager  (FSM)
@@ -175,19 +177,34 @@ class ShareNodeDict(dict):
 class TrashFileShelf(file_shelf.CachedFileShelf):
     """Custom file shelf that supports share and node as keys."""
 
+    _marker_flag = 'marker'
+    _marker_len = len(_marker_flag)
+
     def key_file(self, key):
-        """Supports share and node as keys."""
+        """Support share and node as keys."""
         share_id, node_id = key
 
+        # convert the markers to a string that flags them
+        if IMarker.providedBy(share_id):
+            share_id = str(share_id) + self._marker_flag
+        if IMarker.providedBy(node_id):
+            node_id = str(node_id) + self._marker_flag
+
         # build a string with the node_id first to have a more sparse
         # layout in disk
         key = "%s|%s" % (node_id, share_id)
         return super(TrashFileShelf, self).key_file(key)
 
     def keys(self):
-        """Restores the share/node pair"""
+        """Restore the share/node pair"""
         for key in super(TrashFileShelf, self).keys():
             node_id, share_id = key.split("|")
+            if node_id == 'None':
+                node_id = None
+            elif node_id.endswith(self._marker_flag):
+                node_id = MDMarker(node_id[:-self._marker_len])
+            if share_id.endswith(self._marker_flag):
+                share_id = MDMarker(share_id[:-self._marker_len])
             yield (share_id, node_id)
 
 
@@ -216,6 +233,7 @@ class FileSystemManager(object):
                             type(data_dir))
         fsmdir = os.path.join(data_dir, 'fsm')
         trashdir = os.path.join(data_dir, 'trash')
+        movelimbodir = os.path.join(data_dir, 'move_limbo')
         self.partials_dir = partials_dir
         if not os.path.exists(self.partials_dir):
             os.makedirs(self.partials_dir)
@@ -226,6 +244,8 @@ class FileSystemManager(object):
                                              cache_compact_threshold=4)
         self.trash = TrashFileShelf(trashdir, cache_size=100,
                                     cache_compact_threshold=4)
+        self.move_limbo = TrashFileShelf(movelimbodir, cache_size=100,
+                                         cache_compact_threshold=4)
         self.shares = {}
         self.vm = vm
         self.eq = None  # this will be registered later
@@ -331,6 +351,9 @@ class FileSystemManager(object):
             if mdobj["server_hash"] is None:
                 mdobj["server_hash"] = ""
 
+            # add the generation number (new to v5)
+            mdobj["generation"] = None
+
             # write back the object
             self.fs[mdid] = mdobj
 
@@ -359,6 +382,10 @@ class FileSystemManager(object):
 
             # fix the path
             self._fix_path_for_new_layout(mdobj)
+
+            # add the generation number (new to v5)
+            mdobj["generation"] = None
+
             # write back the object
             self.fs[mdid] = mdobj
 
@@ -385,6 +412,10 @@ class FileSystemManager(object):
 
             # fix the path
             self._fix_path_for_new_layout(mdobj)
+
+            # add the generation number (new to v5)
+            mdobj["generation"] = None
+
             # write back the object
             self.fs[mdid] = mdobj
 
@@ -405,8 +436,34 @@ class FileSystemManager(object):
         for mdid, mdobj in self._safe_fs_iteritems():
             # fix the path
             self._fix_path_for_new_layout(mdobj)
+
+            # add the generation number (new to v5)
+            mdobj["generation"] = None
+
             # write back the object
             self.fs[mdid] = mdobj
+
+            # and of course, load the metadata
+            abspath = self.get_abspath(mdobj["share_id"], mdobj["path"])
+            self._idx_path[abspath] = mdid
+            if mdobj["node_id"] is not None:
+                self._idx_node_id[(mdobj["share_id"], mdobj["node_id"])] = mdid
+
+        # set new version
+        with open(self._version_file, "w") as fh:
+            fh.write(METADATA_VERSION)
+
+    def _load_metadata_4(self, old_version):
+        """Loads metadata from version 4."""
+        logger("loading metadata from old version %r", old_version)
+
+        for mdid, mdobj in self._safe_fs_iteritems():
+            # add the generation number (new to v5)
+            mdobj["generation"] = None
+
+            # write back the object
+            self.fs[mdid] = mdobj
+
             # and of course, load the metadata
             abspath = self.get_abspath(mdobj["share_id"], mdobj["path"])
             self._idx_path[abspath] = mdid
@@ -441,7 +498,7 @@ class FileSystemManager(object):
         relpath = self._share_relative_path(share_id, path)
         newobj = dict(path=relpath, node_id=None, share_id=share_id,
                       is_dir=is_dir, local_hash="", server_hash="",
-                      mdid=mdid)
+                      mdid=mdid, generation=None, crc32=None, size=None)
         newobj["info"] = dict(created=time.time(), is_partial=False)
         # only one stat, (instead of os.path.exists & os.stat)
         newobj["stat"] = get_stat(path)
@@ -586,7 +643,7 @@ class FileSystemManager(object):
         self.set_by_mdid(mdid, **kwargs)
 
     def refresh_stat(self, path):
-        """Refreshes the stat of a md object."""
+        """Refresh the stat of a md object."""
         log_debug("refresh stat to path=%r", path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
@@ -594,7 +651,7 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def move_file(self, new_share_id, path_from, path_to):
-        """Moves a file/dir from one point to the other."""
+        """Move a file/dir from one point to the other."""
         path_from = os.path.normpath(path_from)
         path_to = os.path.normpath(path_to)
         mdid = self._idx_path[path_from]
@@ -621,24 +678,23 @@ class FileSystemManager(object):
         self.moved(new_share_id, path_from, path_to)
 
     def moved(self, new_share_id, path_from, path_to):
-        """change the metadata of a moved file"""
+        """Change the metadata of a moved file."""
         path_from = os.path.normpath(path_from)
         path_to = os.path.normpath(path_to)
-
-        # if the move overwrites other file, adjust *that* metadata
-        if path_to in self._idx_path:
-            mdid = self._idx_path.pop(path_to)
-            mdobj = self.fs[mdid]
-            if mdobj["node_id"] is not None:
-                del self._idx_node_id[(mdobj["share_id"], mdobj["node_id"])]
-            del self.fs[mdid]
-
-        # adjust the metadata of "from" file
         mdid = self._idx_path.pop(path_from)
         log_debug("move_file: mdid=%r path_from=%r path_to=%r",
                                                     mdid, path_from, path_to)
-        mdobj = self.fs[mdid]
 
+        # if the move overwrites other file, send it to trash
+        if path_to in self._idx_path:
+            to_mdid = self._idx_path[path_to]
+            parent_path = os.path.dirname(path_to)
+            parent_mdid = self._idx_path[parent_path]
+            parent_mdobj = self.fs[parent_mdid]
+            self.delete_to_trash(to_mdid, parent_mdobj['node_id'])
+
+        # adjust the metadata of "from" file
+        mdobj = self.fs[mdid]
         self._idx_path[path_to] = mdid
 
         # change the path, make it relative to the share_id
@@ -673,9 +729,8 @@ class FileSystemManager(object):
                 mdobj["path"] = relpath
                 self.fs[mdid] = mdobj
 
-
     def delete_metadata(self, path):
-        """Deletes the metadata."""
+        """Delete the metadata."""
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
@@ -687,10 +742,10 @@ class FileSystemManager(object):
             del self._idx_node_id[(mdobj["share_id"], mdobj["node_id"])]
         del self.fs[mdid]
 
-    def _delete_dir_tree(self, path):
+    def _delete_dir_tree(self, path, share_id):
         """Helper function to recursively remove a non-empty directory.
 
-        Removes the dir if there aren't local changes, and returns True.
+        Remove the dir if there aren't local changes, and return True.
         If there are, return False.
 
         Notifications are disabled for every sub-path within path.
@@ -716,11 +771,25 @@ class FileSystemManager(object):
             filter_name = is_dir and "FS_DIR_DELETE" or "FS_FILE_DELETE"
             self.eq.add_to_mute_filter(filter_name, p)
             self.delete_metadata(p)
-        shutil.rmtree(path)
+
+        # get the share and if it's read-only
+        share = self._get_share(share_id)
+        is_ro_share = not share.can_write()
+        def handle_ro_share(func, subpath, exc_info):
+            """Handle rmtree EACCES error for ro-shares."""
+            if exc_info[0] == OSError and exc_info[1].errno == errno.EACCES \
+               and is_ro_share:
+                # use EnableShareWrite directly to avoid a lot of share
+                # lookups.
+                with EnableShareWrite(share, subpath):
+                    func(subpath)
+            else:
+                raise
+        shutil.rmtree(path, onerror=handle_ro_share)
         return True
 
     def delete_file(self, path):
-        """Deletes a file/dir and the metadata."""
+        """Delete a file/dir and the metadata."""
         # adjust the metadata
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
@@ -741,7 +810,8 @@ class FileSystemManager(object):
         except OSError, e:
             self.eq.rm_from_mute_filter(expected_event, path)
             if e.errno == errno.ENOTEMPTY:
-                if not self._delete_dir_tree(path=path):
+                if not self._delete_dir_tree(path=path,
+                                             share_id=mdobj['share_id']):
                     raise
             else:
                 m = "OSError %s when trying to remove file/dir %r"
@@ -749,7 +819,7 @@ class FileSystemManager(object):
         self.delete_metadata(path)
 
     def move_to_conflict(self, mdid):
-        """Moves a file/dir to its .conflict."""
+        """Move a file/dir to its .conflict."""
         mdobj = self.fs[mdid]
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
         log_debug("move_to_conflict: path=%r mdid=%r", path, mdid)
@@ -793,7 +863,7 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def _check_partial(self, mdid):
-        """Checks consistency between internal flag and FS regarding partial"""
+        """Check consistency between internal flag and FS regarding partial"""
         # get the values
         mdobj = self.fs[mdid]
         partial_in_md = mdobj["info"]["is_partial"]
@@ -814,7 +884,7 @@ class FileSystemManager(object):
         return partial_path + '.' + filename
 
     def create_partial(self, node_id, share_id):
-        """Creates a .partial in disk and set the flag in metadata."""
+        """Create a .partial in disk and set the flag in metadata."""
         mdid = self._idx_node_id[(share_id, node_id)]
         log_debug("create_partial: mdid=%r share_id=%r node_id=%r",
                   mdid, share_id, node_id)
@@ -842,18 +912,15 @@ class FileSystemManager(object):
     def get_partial_for_writing(self, node_id, share_id):
         """Get a write-only fd to a partial file"""
         mdid = self._idx_node_id[(share_id, node_id)]
-        log_debug("get_partial_for_writing: mdid=%r share_id=%r node_id=%r" %
-                  (mdid, share_id, node_id))
-        if not self._check_partial(mdid):
-            raise ValueError("The object with share_id %r and node_id %r is "
-                             "not partial!" % (share_id, node_id))
+        log_debug("get_partial_for_writing: mdid=%r share_id=%r node_id=%r",
+                  mdid, share_id, node_id)
 
         mdobj = self.fs[mdid]
         partial_path = self._get_partial_path(mdobj)
         return open(partial_path, "w")
 
     def get_partial(self, node_id, share_id):
-        """Gets a read-only fd to a partial file."""
+        """Get a read-only fd to a partial file."""
         mdid = self._idx_node_id[(share_id, node_id)]
         if not self._check_partial(mdid):
             raise ValueError("The object with share_id %r and node_id %r is "
@@ -864,7 +931,7 @@ class FileSystemManager(object):
         return fd
 
     def commit_partial(self, node_id, share_id, local_hash):
-        """Creates a .partial in disk and set the flag in metadata."""
+        """Create a .partial in disk and set the flag in metadata."""
         mdid = self._idx_node_id[(share_id, node_id)]
         mdobj = self.fs[mdid]
         if mdobj["is_dir"]:
@@ -890,7 +957,7 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def remove_partial(self, node_id, share_id):
-        """Removes a .partial in disk and set the flag in metadata."""
+        """Remove a .partial in disk and set the flag in metadata."""
         mdid = self._idx_node_id[(share_id, node_id)]
 
         # delete the .partial, and set the md info
@@ -912,14 +979,14 @@ class FileSystemManager(object):
         self.fs[mdid] = mdobj
 
     def upload_finished(self, mdid, server_hash):
-        """Sets the metadata with timestamp and server hash."""
+        """Set the metadata with timestamp and server hash."""
         mdobj = self.fs[mdid]
         mdobj["info"]["last_uploaded"] = time.time()
         mdobj["server_hash"] = server_hash
         self.fs[mdid] = mdobj
 
     def _get_mdid_from_args(self, kwargs, parent):
-        """Parses the kwargs and gets the mdid."""
+        """Parse the kwargs and gets the mdid."""
         if len(kwargs) == 1 and "path" in kwargs:
             path = os.path.normpath(kwargs["path"])
             return self._idx_path[path]
@@ -996,7 +1063,7 @@ class FileSystemManager(object):
         return has_changed
 
     def dir_content(self, path):
-        """Returns the content of the directory in a server-comparable way."""
+        """Return the content of the directory in a server-comparable way."""
         path = os.path.normpath(path)
         mdid = self._idx_path[path]
         mdobj = self.fs[mdid]
@@ -1014,7 +1081,7 @@ class FileSystemManager(object):
         return sorted(_get_all())
 
     def open_file(self, mdid):
-        """returns a file like object for reading the contents of the file."""
+        """Return a file like object for reading the contents of the file."""
         mdobj = self.fs[mdid]
         if mdobj["is_dir"]:
             raise ValueError("You can only open files, not directories.")
@@ -1022,7 +1089,7 @@ class FileSystemManager(object):
         return open(self.get_abspath(mdobj['share_id'], mdobj['path']))
 
     def _share_relative_path(self, share_id, path):
-        """ returns the relative path from the share_id. """
+        """Return the relative path from the share_id."""
         share = self._get_share(share_id)
         if path == share.path:
             # the relaitve path is the fullpath
@@ -1036,7 +1103,7 @@ class FileSystemManager(object):
         return relpath.lstrip(os.path.sep)
 
     def _get_share(self, id):
-        """ returns the share/udf with share or volume id: id. """
+        """Returns the share/udf with share or volume id: id."""
         # TODO: refactor fsm to use volume instead of share
         share = self.shares.get(id, None)
         if share is None:
@@ -1045,7 +1112,7 @@ class FileSystemManager(object):
         return share
 
     def get_abspath(self, share_id, path):
-        """ return the absolute path: share.path + path"""
+        """Return the absolute path: share.path + path."""
         share_path = self._get_share(share_id).path
         if share_path == path:
             # the relaitve path is the fullpath
@@ -1054,7 +1121,7 @@ class FileSystemManager(object):
             return os.path.join(share_path, path)
 
     def _enable_share_write(self, share_id, path):
-        """ helper to create a EnableShareWrite context manager. """
+        """Helper to create a EnableShareWrite context manager."""
         share = self._get_share(share_id)
         return EnableShareWrite(share, path)
 
@@ -1086,38 +1153,144 @@ class FileSystemManager(object):
         """Move the node to the trash."""
         mdobj = self.fs[mdid]
         node_id = mdobj["node_id"]
+        if node_id is None:
+            node_id = MDMarker(mdid)
         share_id = mdobj["share_id"]
         path = self.get_abspath(mdobj['share_id'], mdobj['path'])
         log_debug("delete_to_trash: mdid=%r, parent=%r, share=%r, node=%r, "
                   "path=%r", mdid, parent_id, share_id, node_id, path)
         self.delete_metadata(path)
-        self.trash[(share_id, node_id)] = (mdid, parent_id)
+        self.trash[(share_id, node_id)] = (mdid, parent_id, path)
 
     def remove_from_trash(self, share_id, node_id):
-        """Deletes the node from the trash."""
+        """Delete the node from the trash."""
         log_debug("remove_from_trash: share=%r, node=%r", share_id, node_id)
         if (share_id, node_id) in self.trash:
             del self.trash[(share_id, node_id)]
 
+    def node_in_trash(self, share_id, node_id):
+        """Return if the node is in the trash."""
+        return (share_id, node_id) in self.trash
+
     def get_iter_trash(self):
         """Return the trash element by element."""
-        for (share_id, node_id), (mdid, parent_id) in self.trash.iteritems():
-            yield share_id, node_id, parent_id
+        for (share_id, node_id), node_info in self.trash.iteritems():
+            yield share_id, node_id, node_info[1]
+
+    def get_dirty_nodes(self):
+        """Return the mdid of the dirty nodes, one by one."""
+        for _, v in self.fs.items():
+            if v.get('dirty'):
+                yield _MDObject(**v)
+
+    def add_to_move_limbo(self, share_id, node_id, old_parent_id,
+                          new_parent_id, new_name):
+        """Add the operation info to the move limbo."""
+        log_debug("add to move limbo: share=%r, node=%r, old_parent=%r, "
+                  "new_parent=%r, new_name=%r", share_id, node_id,
+                  old_parent_id, new_parent_id, new_name)
+        self.move_limbo[(share_id, node_id)] = (old_parent_id,
+                                                new_parent_id, new_name)
+
+    def remove_from_move_limbo(self, share_id, node_id):
+        """Remove the node from the move limbo."""
+        log_debug("remove from move limbo: share=%r, node=%r",
+                  share_id, node_id)
+        if (share_id, node_id) in self.move_limbo:
+            del self.move_limbo[(share_id, node_id)]
+
+    def get_iter_move_limbo(self):
+        """Return the move limbo node by node."""
+        for k, v in self.move_limbo.iteritems():
+            share_id, node_id = k
+            old_parent_id, new_parent_id, new_name = v
+            yield share_id, node_id, old_parent_id, new_parent_id, new_name
+
+    def make_dir(self, mdid):
+        """Create the dir in disk."""
+        mdobj = self.get_by_mdid(mdid)
+        if not mdobj.is_dir:
+            raise ValueError("make_dir must be on a file (mdid: %r)" % (mdid,))
+
+        full_path = self.get_abspath(mdobj.share_id, mdobj.path)
+        with self._enable_share_write(mdobj.share_id, full_path) as enable:
+            if not enable.ro:
+                self.eq.add_to_mute_filter('FS_DIR_CREATE', full_path)
+
+            try:
+                os.mkdir(full_path)
+            except OSError, e:
+                if not e.errno == 17: #already exists
+                    raise
+
+            if not enable.ro:
+                # add the watch: we hope the user wont have time to add a file
+                # just after *we* created the directory; see bug #373940
+                self.eq.inotify_add_watch(full_path)
+
+    def dereference_ok_limbos(self, marker, value):
+        """Dereference markers in the limbos with a value."""
+        for (share, node), (mdid, parent, path) in self.trash.iteritems():
+            if node == marker:
+                del self.trash[(share, node)]
+                self.trash[(share, value)] = (mdid, parent, path)
+                log_debug("dereference ok trash: share=%r  marker=%r  "
+                          "new node=%r", share, marker, value)
+            elif parent == marker:
+                self.trash[(share, node)] = (mdid, value, path)
+                log_debug("dereference ok trash: share=%r  node=%r  marker=%r"
+                          "  new parent=%r", share, node, marker, value)
+
+        for k, v in self.move_limbo.iteritems():
+            share, node = k
+            old_parent, new_parent, new_name = v
+
+            if node == marker:
+                del self.move_limbo[(share, node)]
+                self.move_limbo[(share, value)] = v
+                log_debug("dereference ok move limbo: share=%r  marker=%r  "
+                          "new node=%r", share, marker, value)
+            else:
+                # both parents can be the same marker at the same time
+                if old_parent == marker or new_parent == marker:
+                    if old_parent == marker:
+                        old_parent = value
+                    if new_parent == marker:
+                        new_parent = value
+                log_debug("dereference ok move limbo: share=%r  node=%r  "
+                          "marker=%r  old_parent=%r  new_parent=%r",
+                          share, node, marker, old_parent, new_parent)
+                self.move_limbo[k] = (old_parent, new_parent, new_name)
+
+    def dereference_err_limbos(self, marker):
+        """Dereference markers in the limbos with an error.
+
+        As the dependency is not valid, we just remove the item.
+        """
+        for (share, node), (_, parent, _) in self.trash.iteritems():
+            if node == marker or parent == marker:
+                log_debug("dereference err trash: share=%r  node=%r  "
+                          "marker=%r", share, node, marker)
+                del self.trash[(share, node)]
+
+        move_items = self.move_limbo.iteritems()
+        for (share, node), (old_parent, new_parent, _) in move_items:
+            if node == marker or old_parent == marker or new_parent == marker:
+                log_debug("dereference err move limbo: share=%r  node=%r  "
+                          "marker=%r", share, node, marker)
+                del self.move_limbo[(share, node)]
 
 
 class EnableShareWrite(object):
-    """ Context manager to allow write in ro-shares. """
-    #pylint: disable-msg=W0201
+    """Context manager to allow write in ro-shares."""
 
     def __init__(self, share, path):
-        """ create the instance """
         self.share = share
         self.path = path
-
+        self.ro = not self.share.can_write()
 
     def __enter__(self):
-        """ ContextManager API """
-        self.ro = not self.share.can_write()
+        """ContextManager API."""
         path_stat = get_stat(self.path)
         if path_stat is not None and stat.S_ISDIR(path_stat.st_mode) and \
            os.path.normpath(self.path) == os.path.normpath(self.share.path):
@@ -1137,9 +1310,10 @@ class EnableShareWrite(object):
             path_stat = get_stat(self.path)
             if path_stat is not None and not stat.S_ISDIR(path_stat.st_mode):
                 os.chmod(self.path, 0744)
+        return self
 
     def __exit__(self, *exc_info):
-        """ ContextManager API """
+        """ContextManager API."""
         if self.ro:
             path_stat = get_stat(self.path)
             if path_stat is not None:
@@ -1152,7 +1326,8 @@ class EnableShareWrite(object):
 
 
 def get_stat(path):
-    """Returns os.lstat or None if errno == ENOENT.
+    """Return os.lstat or None if errno == ENOENT.
+
     os.lstat is used as we don't support symlinks
     """
     try:
