@@ -27,12 +27,12 @@ import os
 
 from twisted.internet import reactor
 
-from ubuntuone.storageprotocol.hash import \
+from ubuntuone.storageprotocol.content_hash import \
     content_hash_factory, crc32
 
 
 class StopHashing(Exception):
-    """The current hash was cancelled"""
+    """The current hash was cancelled."""
 
 
 class _Hasher(threading.Thread):
@@ -77,8 +77,11 @@ class _Hasher(threading.Thread):
             except StopHashing, e:
                 self.logger.debug(str(e))
             else:
-                m = "Hasher: path hash pushed:  path %r  hash %r"
-                self.logger.debug(m, path, result)
+                hashdata, crc, size, stat = result
+                self.logger.debug("Hasher: path hash pushed:  path=%r  hash=%s"
+                                  "  crc=%s  size=%d  st_ino=%d  st_size=%d"
+                                  "  st_mtime=%r", path, hashdata,crc, size,
+                                  stat.st_ino, stat.st_size, stat.st_mtime)
                 reactor.callFromThread(self.eq.push,
                                        "HQ_HASH_NEW", path, *result)
             finally:
@@ -88,8 +91,10 @@ class _Hasher(threading.Thread):
             self.queue.task_done()
 
     def stop(self):
-        """Stop the hasher (will be effective in the next loop if a hash
-        is in progress)
+        """Stop the hasher.
+
+        Will be effective in the next loop if a hash is in progress.
+
         """
         # clear the queue to push a end_mark, just to unblok if we are waiting
         # for a new item
@@ -125,12 +130,12 @@ class _Hasher(threading.Thread):
         return hasher.content_hash(), crc, size, initial_stat
 
     def busy(self):
-        """return whether we are busy"""
+        """Return whether we are busy."""
         with self.mutex:
             return self.hashing
 
     def cancel_if_running(self, path):
-        """request a cancel/stop of the current hash, if it's == path"""
+        """Request a cancel/stop of the current hash, if it's == path."""
         with self.mutex:
             if self.hashing == path:
                 self._should_cancel = path
