@@ -93,18 +93,6 @@ class IMetaQueue(Interface):
         Unlink the given node.
         """
 
-    def query(items):
-        """
-        Gossip with the server about the freshness of the server hashes
-
-        Items is a list of (share_id, node_id, node_hash).
-        """
-
-    def listdir(share_id, node_id, server_hash, fileobj):
-        """
-        List (get the content of) the given directory.
-        """
-
     def list_shares():
         """
         Get a list of the shares, and put the result on the event queue.
@@ -130,15 +118,13 @@ class IActionQueue(IContentQueue, IMetaQueue):
     """
 
     def connect(host, port, use_ssl=False):
-        """
-        Open a (possibly SSL) connection to the API server on (host,
-        port). Once you've connected, do the OAuth dance.
+        """Open a (possibly SSL) connection to the API server on (host, port).
+
+        Once you've connected, authenticate.
         """
 
     def disconnect():
-        """
-        Close the connection
-        """
+        """Close the connection."""
 
 class IMarker(Interface):
     """
