@@ -57,11 +57,24 @@ class LocalRescan(object):
         self._queue = collections.deque()
         self._previous_deferred = None
 
+    @defer.inlineCallbacks
     def start(self):
         """Start the comparison."""
         log_info("start scan all volumes")
         to_scan = self._get_volumes(all_volumes=False)
         for vol in to_scan:
+            # check that the path exists in disk
+            if not os.path.exists(vol.path):
+                log_warning('Volume dissapeared: %r - %r', vol.volume_id, vol.path)
+                if isinstance(vol, volume_manager.Share):
+                    log_debug('Removing %r metadata', vol.volume_id)
+                    self.vm.share_deleted(vol.volume_id)
+                elif isinstance(vol, volume_manager.UDF):
+                    log_debug('Unsubscribing %r', vol.volume_id)
+                    self.vm.unsubscribe_udf(vol.volume_id)
+                # if root is missing, we should crash and burn as it's
+                # created on each startup!
+                continue
             try:
                 mdobj = self.fsm.get_by_path(vol.path)
             except KeyError:
@@ -70,26 +83,92 @@ class LocalRescan(object):
                 # download it again
                 self.vm.share_deleted(vol.volume_id)
             else:
-                self._queue.appendleft((vol, vol.path, True,
-                                        mdobj.mdid, False))
-        d = self._queue_scan()
-        d.addCallback(self._process_trash)
-        return d
+                self._queue.appendleft((vol, vol.path, mdobj.mdid, False))
 
-    def _process_trash(self, _):
-        """Process the trash of FSM and send AQ orders to unlink the nodes."""
+        # first of all, remove old partials and clean trash
+        self._remove_partials()
+        self._process_limbo()
+        self._process_ro_shares()
+
+        yield self._queue_scan()
+        self._show_broken_nodes()
+
+    def _remove_partials(self):
+        """Remove all old .partial files.
+
+        As LR now issues new downloads, and doesn't depend on these file to
+        fix the nodes states, we just remove them to don't let garbage behind.
+        """
+        try:
+            partials = os.listdir(self.fsm.partials_dir)
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+            # no partials dir at all
+            return
+
+        for partial in partials:
+            partial_path = os.path.join(self.fsm.partials_dir, partial)
+            log_debug("Removing old .partial: %r", partial_path)
+            os.remove(partial_path)
+
+    def _process_limbo(self):
+        """Process the FSM limbos and send corresponding AQ orders."""
         log_info("processing trash")
+        trash_log = "share_id=%r  parent_id=%r  node_id=%r"
         for share_id, node_id, parent_id in self.fsm.get_iter_trash():
-            datalog = "share_id: %r   parent_id: %r   node_id: %r" % (
-                      share_id, parent_id, node_id)
-            if node_id is None or IMarker.providedBy(parent_id):
+            datalog = trash_log % (share_id, parent_id, node_id)
+            if IMarker.providedBy(node_id) or IMarker.providedBy(parent_id):
                 # situation where the node is not in the server
                 log_info("removing from trash: " + datalog)
                 self.fsm.remove_from_trash(share_id, node_id)
                 continue
-            log_info("unlink from trash: " + datalog)
+            log_info("generating Unlink from trash: " + datalog)
             self.aq.unlink(share_id, parent_id, node_id)
 
+        log_info("processing move limbo")
+        move_log = ("share_id=%r  node_id=%r  old_parent_id=%r  "
+                    "new_parent_id=%r  new_name=%r")
+        for data in self.fsm.get_iter_move_limbo():
+            to_log = move_log % data
+            share_id, node_id, old_parent_id, new_parent_id, new_name = data
+            maybe_markers = (share_id, node_id, old_parent_id, new_parent_id)
+            if any(IMarker.providedBy(x) for x in maybe_markers):
+                # situation where the move was not ready
+                log_info("removing from move limbo: " + to_log)
+                self.fsm.remove_from_move_limbo(share_id, node_id)
+                continue
+            log_info("generating Move from limbo: " + to_log)
+            self.aq.move(share_id, node_id, old_parent_id,
+                         new_parent_id, new_name)
+
+    def _process_ro_shares(self):
+        """Process ro shares and reschedule interrupted downloads."""
+        # to avoid the lookups in the nested for
+        changed = self.fsm.changed
+        CHANGED_SERVER = self.fsm.CHANGED_SERVER
+        for share in self._get_volumes(all_volumes=False, access_level="View"):
+            for mdobj in self.fsm.get_mdobjs_by_share_id(share.id):
+                if changed(mdid=mdobj.mdid) == CHANGED_SERVER:
+                    fullname = os.path.join(share.path, mdobj.path)
+                    if mdobj.is_dir:
+                        # old state, no sense now with generations
+                        # but required for the migration path.
+                        log_warning("Found a directory in SERVER: %r", fullname)
+                        mdobj = self.fsm.get_by_path(fullname)
+                        self.fsm.set_by_mdid(mdobj.mdid,
+                                             server_hash=mdobj.local_hash)
+                        self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
+                    else:
+                        log_debug("comp yield: file %r in SERVER", fullname)
+                        self._resume_download(fullname)
+
+    def _show_broken_nodes(self):
+        """Log the broken nodes."""
+        for mdobj in self.fsm.get_dirty_nodes():
+            m = "Broken node: volume_id=%r node_id=%r mdid=%r path=%r"
+            log_info(m, mdobj.share_id, mdobj.node_id, mdobj.mdid, mdobj.path)
+
     def _get_volumes(self, all_volumes=True, access_level='Modify'):
         """Get all the shares and udfs matching access_level to compare."""
         modify = access_level == 'Modify'
@@ -132,7 +211,7 @@ class LocalRescan(object):
             raise ValueError(m)
 
         # No, 'share' is surely defined; pylint: disable-msg=W0631
-        self._queue.appendleft((share, direct, False, mdid, udfmode))
+        self._queue.appendleft((share, direct, mdid, udfmode))
         return self._queue_scan()
 
     def _send_scan_error(self, mdid, udfmode):
@@ -218,7 +297,7 @@ class LocalRescan(object):
                                obj.node_id, obj.local_hash, obj.server_hash))
         return share_info
 
-    def _scan_tree(self, share, path, scan_partials, mdid, udfmode):
+    def _scan_tree(self, share, path, mdid, udfmode):
         """Scans a whole tree, using the received path as root."""
         log_debug("_scan_tree:  share_path: %r  path: %r", share.path, path)
 
@@ -226,8 +305,7 @@ class LocalRescan(object):
             """Explore into the subdirs."""
             for direct in newdirs:
                 log_debug("explore subdir: %r", direct)
-                self._queue.appendleft((share, direct, scan_partials,
-                                        mdid, udfmode))
+                self._queue.appendleft((share, direct, mdid, udfmode))
 
         def re_launch(failure):
             """Explore that directory again."""
@@ -235,8 +313,7 @@ class LocalRescan(object):
                 reason = failure.getErrorMessage()
                 log_debug("re queue, transaction dirty for %r, reason: %s",
                                                                   path, reason)
-                self._queue.appendleft((share, path, scan_partials,
-                                        mdid, udfmode))
+                self._queue.appendleft((share, path, mdid, udfmode))
             elif failure.check(OSError, IOError):
                 reason = failure.getErrorMessage()
                 m = "Disk error while scanning path %r, reason: %s"
@@ -252,13 +329,49 @@ class LocalRescan(object):
                           failure.type, failure.value, failure.getTraceback())
                 return failure
 
-        d = defer.succeed((share, path, scan_partials, udfmode))
+        d = defer.succeed((share, path, udfmode))
         d.addCallbacks(self._scan_one_dir)
         d.addCallbacks(go_deeper, re_launch)
         d.addCallback(self._process_next_queue)
         return d
 
-    def _compare(self, dirpath, dirnames, filenames, share, scan_partials):
+    def _resume_download(self, fullname):
+        """Resume an interrupted download."""
+        mdobj = self.fsm.get_by_path(fullname)
+        fh_factory = lambda: self.fsm.get_partial_for_writing(mdobj.node_id,
+                                                              mdobj.share_id)
+        self.aq.download(mdobj.share_id, mdobj.node_id,
+                         mdobj.server_hash, fh_factory)
+
+    def _resume_upload(self, fullname):
+        """Resume an interrupted upload."""
+        mdobj = self.fsm.get_by_path(fullname)
+        fh_factory = lambda: self.fsm.open_file(mdobj.mdid)
+        self.aq.upload(mdobj.share_id, mdobj.node_id, mdobj.server_hash,
+                       mdobj.local_hash, mdobj.crc32, mdobj.size, fh_factory)
+
+    def check_stat(self, fullname, oldstat):
+        """Check stat info and return if different.
+
+        Don't compare the full stat, only what is relevant:
+
+        - st_ino: the data location changed in disk, may be something else
+        - st_size: it changed in size, surely different content
+        - st_mtime: the content could be different even having the same size
+        """
+        newstat = os.stat(fullname)
+        different = (newstat.st_ino != oldstat.st_ino or
+                     newstat.st_size != oldstat.st_size or
+                     newstat.st_mtime != oldstat.st_mtime)
+        if different:
+            log_debug("stat differ for: %r  "
+                      "Old: st_ino=%d st_size=%d st_mtime=%r  "
+                      "New: st_ino=%d st_size=%d st_mtime=%r", fullname,
+                      oldstat.st_ino, oldstat.st_size, oldstat.st_mtime,
+                      newstat.st_ino, newstat.st_size, newstat.st_mtime)
+        return different
+
+    def _compare(self, dirpath, dirnames, filenames, share):
         """Compare the directories with the info that should be there."""
         log_debug("comparing directory %r", dirpath)
 
@@ -291,14 +404,6 @@ class LocalRescan(object):
                     if e.errno != errno.ENOENT:
                         raise
 
-
-        def check_stat(fullname, statinfo):
-            """Generate event if stats differ."""
-            newstat = os.stat(fullname)
-            if statinfo != newstat:
-                log_debug("stat differ for: %r", fullname)
-                events.append(('FS_FILE_CLOSE_WRITE', fullname))
-
         # check all directories
         to_scan_later = []
         events = []
@@ -330,29 +435,29 @@ class LocalRescan(object):
                     log_info("comp yield: file %r became a dir!", fullname)
                     events.append(('FS_FILE_DELETE', fullname))
                     events.append(('FS_DIR_CREATE', fullname))
+                elif changed == "SERVER":
+                    # old state, no sense now with generations
+                    log_warning("Found a directory in SERVER: %r", fullname)
+                    mdobj = self.fsm.get_by_path(fullname)
+                    self.fsm.set_by_mdid(mdobj.mdid,
+                                         server_hash=mdobj.local_hash)
+                    self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
+                    to_scan_later.append(fullname)
+                elif changed == "NONE":
+                    # it's old, we should scan it later
+                    log_debug("comp yield: dir %r will be scaned later "
+                              "because it's in NONE!", fullname)
+                    to_scan_later.append(fullname)
                 else:
-                    if changed == "SERVER":
-                        # download interrupted
-                        log_debug("comp yield: dir %r in SERVER", fullname)
-                        mdobj = self.fsm.get_by_path(fullname)
-                        self.fsm.set_by_mdid(mdobj.mdid,
-                                             server_hash=mdobj.local_hash)
-                        self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
-                        to_scan_later.append(fullname)
-                    elif changed == "NONE":
-                        # it's old, we should scan it later
-                        log_debug("comp yield: dir %r will be scaned later because is in NONE!",
-                                                                      fullname)
-                        to_scan_later.append(fullname)
-                    else:
-                        m = "Wrong 'changed' value for %r: " + changed
-                        despair(m, fullname, also_children=True)
+                    m = "Wrong 'changed' value for %r: " + changed
+                    despair(m, fullname, also_children=True)
 
             else:
                 # hey, it's new!
                 log_debug("comp yield: directory %r is new!", fullname)
                 events.append(('FS_DIR_CREATE', fullname))
 
+
         # check all files
         for fname in filenames:
             fullname = os.path.join(dirpath, fname)
@@ -364,130 +469,39 @@ class LocalRescan(object):
                     events.append(('FS_DIR_DELETE', fullname))
                     events.append(('FS_FILE_CREATE', fullname))
                     events.append(('FS_FILE_CLOSE_WRITE', fullname))
-                else:
-                    if changed == "LOCAL":
-                        # upload interrupted
-                        log_debug("comp yield: file %r in LOCAL state!",
-                                                                    fullname)
+                elif changed == "LOCAL":
+                    log_info("Resuming upload because it was interrupted: %r",
+                             fullname)
+                    self._resume_upload(fullname)
+                elif changed == "NONE":
+                    # what about stat info?
+                    log_debug("comp yield: file %r was here, let's check stat",
+                              fullname)
+                    different = self.check_stat(fullname, statinfo)
+                    if different:
+                        events.append(('FS_FILE_CLOSE_WRITE', fullname))
+                    # no 'else' here: the file is the same as before, all ok
+                elif changed == "SERVER":
+                    log_debug("comp yield: file %r in SERVER", fullname)
+                    different = self.check_stat(fullname, statinfo)
+                    if different:
+                        # hash it to see the changes, Sync will take care
                         events.append(('FS_FILE_CLOSE_WRITE', fullname))
-                    elif changed == "NONE":
-                        # what about stat info?
-                        log_debug("comp yield: file %r was here, let's do check_stat",
-                                                                    fullname)
-                        check_stat(fullname, statinfo)
-                    elif changed == "SERVER":
-                        log_debug("comp yield: file %r in SERVER", fullname)
-                        mdobj = self.fsm.get_by_path(fullname)
-                        self.fsm.set_by_mdid(mdobj.mdid,
-                                             server_hash=mdobj.local_hash)
-                        self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
-                        check_stat(fullname, statinfo)
                     else:
-                        m = "Wrong 'changed' value for %r: " + changed
-                        despair(m, fullname)
-
-            else:
-                # This is kept as is in order to be backward compatible in the
-                # first Local rescan after the upgrade to the new version with
-                # partials in a separate directory
-                if fname.startswith(".u1partial.") or fname == '.u1partial':
-                    # a partial file! it can be a standard file, or the one
-                    # inside a dir (which will be deleted in that case)
-                    if fname == '.u1partial':
-                        # the one in the dir
-                        realname = ''
-                        realfullname = dirpath
-                    else:
-                        realname = fname[11:]
-                        realfullname = os.path.join(dirpath, realname)
-                    if realname not in shouldbe:
-                        # this is the case of a .partial with no md at all!
-                        m = "Found a .partial (%r) with no MD, removing! (1)"
-                        log_debug(m, fullname)
-                        os.remove(fullname)
-                        continue
-
-                    is_dir, statinfo, changed = shouldbe.pop(realname)
-                    if is_dir:
-                        m = ".partial of a file that MD says it's a dir: %r"
-                        despair(m, realfullname, also_remove=fullname)
-                    elif changed != "SERVER":
-                        m = ".partial of a file that 'changed' != SERVER: %r"
-                        despair(m, realfullname, also_remove=fullname)
-                    else:
-                        # download interrupted
-                        m = "comp yield: file %r in SERVER state!"
-                        log_debug(m, fullname)
-                        mdobj = self.fsm.get_by_path(realfullname)
-                        self.fsm.set_by_mdid(mdobj.mdid,
-                                             server_hash=mdobj.local_hash)
-                        self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
-                        check_stat(fullname, statinfo)
-
+                        # file didn't change, resume download
+                        self._resume_download(fullname)
                 else:
-                    # hey, it's new!
-                    log_debug("comp yield: file %r is new!", fullname)
-                    events.append(('FS_FILE_CREATE', fullname))
-
-                    # even if it's empty, we signal to get the hash
-                    # otherwise it will never get "empty" to the server
-                    events.append(('FS_FILE_CLOSE_WRITE', fullname))
-
-
-        if scan_partials:
-            # get the partials info before sync starts creating new partials
-            try:
-                partials = os.listdir(self.fsm.partials_dir)
-            except OSError, e:
-                if e.errno == errno.ENOENT:
-                    partials = set()
-                else:
-                    raise
-        else:
-            partials = []
-        # now check the partials (use a copy of the partials list)
-        for partial in partials:
-            partial_path = os.path.join(self.fsm.partials_dir, partial)
-            # a partial file! it can be a standard file, or the one
-            # inside a dir (which will be deleted in that case)
-            mdid, realname = partial.split('.u1partial.')
-            try:
-                mdobj = self.fsm.get_by_mdid(mdid)
-            except KeyError:
-                # this is the case of a .partial with no md at all!
-                m = "Found a .partial (%r) with no MD, removing! (2)"
-                log_debug(m, partial_path)
-                os.remove(partial_path)
-                continue
+                    m = "Wrong 'changed' value for %r: " + changed
+                    despair(m, fullname)
             else:
-                realfullname = self.fsm.get_abspath(mdobj.share_id, mdobj.path)
+                # hey, it's new!
+                log_debug("comp yield: file %r is new!", fullname)
+                events.append(('FS_FILE_CREATE', fullname))
 
-            if os.path.dirname(realfullname) != dirpath:
-                # if this partial isn't for the current dirpath, ignore it!
-                continue
-            if realfullname == dirpath:
-                realname = ''
+                # even if it's empty, we signal to get the hash
+                # otherwise it will never get "empty" to the server
+                events.append(('FS_FILE_CLOSE_WRITE', fullname))
 
-            if realname not in shouldbe:
-                # this is the case of a .partial with no md at all!
-                m = "Found a .partial (%r) with no MD, removing! (3)"
-                despair(m, realfullname, also_remove=partial_path)
-                continue
-
-            is_dir, statinfo, changed = shouldbe.pop(realname)
-            if is_dir:
-                m = ".partial of a file that MD says it's a dir: %r"
-                despair(m, realfullname, also_remove=partial_path)
-            elif changed != "SERVER":
-                m = ".partial of a file that 'changed' != SERVER: %r"
-                despair(m, realfullname, also_remove=partial_path)
-            else:
-                # download interrupted
-                m = "comp yield: file %r with partial in SERVER state!"
-                log_debug(m, realfullname)
-                self.fsm.set_by_mdid(mdobj.mdid,
-                                     server_hash=mdobj.local_hash)
-                self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
 
         # all these don't exist anymore
         for name, (is_dir, statinfo, changed) in shouldbe.iteritems():
@@ -529,11 +543,8 @@ class LocalRescan(object):
                 if changed == 'SERVER':
                     # download interruped and partial lost
                     log_debug("comp yield: file %r not in disk, in SERVER "
-                              "state, without partial!", fullname)
-                    mdobj = self.fsm.get_by_path(fullname)
-                    self.fsm.set_by_mdid(mdobj.mdid,
-                                         server_hash=mdobj.local_hash)
-                    self.fsm.remove_partial(mdobj.node_id, mdobj.share_id)
+                              "state", fullname)
+                    self._resume_download(fullname)
                 elif changed in ('NONE', 'LOCAL'):
                     # if it had content somewhen, now is really gone (otherwise
                     # it was never really created in the disk)
@@ -582,7 +593,7 @@ class LocalRescan(object):
 
     def _scan_one_dir(self, scan_info):
         """Gets one dir and compares with fsm."""
-        share, dirpath, partials, udf_mode = scan_info
+        share, dirpath, udf_mode = scan_info
 
         if self.eq.inotify_has_watch(dirpath):
             log_debug("Path already has watch: %r", dirpath)
@@ -621,7 +632,7 @@ class LocalRescan(object):
                                 fullname)
 
             events, to_scan_later = self._compare(dirpath, dnames, fnames,
-                                                  share, partials)
+                                                  share)
             to_later.extend(to_scan_later)
             return events
 
