@@ -16,17 +16,17 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-""" SyncDaemon Main"""
-import dbus
-from dbus.mainloop.glib import DBusGMainLoop
-import gnomekeyring
+"""SyncDaemon Main."""
+
 import logging
 import os
 import sys
 
+import dbus
+
+from dbus.mainloop.glib import DBusGMainLoop
 from twisted.internet import defer, reactor, task
 
-from oauth import oauth
 from ubuntuone.syncdaemon import (
     action_queue,
     config,
@@ -72,31 +72,21 @@ class WaitingHelpingHandler(object):
 
 
 class Main(object):
-    """ The one who executes the syncdaemon """
+    """The one who executes the syncdaemon."""
 
     def __init__(self, root_dir, shares_dir, data_dir, partials_dir,
                  host='fs-1.one.ubuntu.com', port=443, dns_srv=None, ssl=True,
-                 disable_ssl_verify=False,
-                 realm='https://ubuntuone.com', glib_loop=False,
-                 mark_interval=120, dbus_events=False,
-                 handshake_timeout=30,
+                 disable_ssl_verify=False, glib_loop=False,
+                 mark_interval=120, dbus_events=False, handshake_timeout=30,
                  shares_symlink_name='Shared With Me',
                  read_limit=None, write_limit=None, throttling_enabled=False,
                  ignore_files=None):
-        """ create the instance. """
         self.root_dir = root_dir
         self.shares_dir = shares_dir
         self.shares_dir_link = os.path.join(self.root_dir, shares_symlink_name)
         self.data_dir = data_dir
         self.partials_dir = partials_dir
         self.logger = logging.getLogger('ubuntuone.SyncDaemon.Main')
-        self.host = host
-        self.port = port
-        self.dns_srv = dns_srv
-        self.ssl = ssl
-        self.disable_ssl_verify = disable_ssl_verify
-        self.realm = realm
-        self.token = None
         user_config = config.get_user_config()
         if read_limit is None:
             read_limit = user_config.get_throttling_read_limit()
@@ -107,11 +97,9 @@ class Main(object):
 
         self.vm = volume_manager.VolumeManager(self)
         self.fs = filesystem_manager.FileSystemManager(data_dir,
-                                                       self.partials_dir,
-                                                       self.vm)
+                                                       partials_dir, self.vm)
         self.event_q = event_queue.EventQueue(self.fs, ignore_files)
         self.fs.register_eq(self.event_q)
-        self.oauth_client = OAuthClient(self.realm)
 
         # subscribe VM to EQ
         self.event_q.subscribe(self.vm)
@@ -120,7 +108,7 @@ class Main(object):
         # we don't have the oauth tokens yet, we 'll get them later
         self.action_q = action_queue.ActionQueue(self.event_q, self,
                                                  host, port,
-                                                 self.dns_srv, ssl,
+                                                 dns_srv, ssl,
                                                  disable_ssl_verify,
                                                  read_limit, write_limit,
                                                  throttling_enabled)
@@ -150,39 +138,37 @@ class Main(object):
         self.mark.start(mark_interval)
 
     def log_mark(self):
-        """ log a "mark" that includes the current AQ state and queue size"""
-        self.logger.note("%s %s (state: %s; queues: metadata: %d; content: %d;"
-                         " hash: %d, fsm-cache: hit=%d miss=%d) %s" % ('-'*4,
-                                         'MARK', self.state_manager,
-                                         len(self.action_q.meta_queue),
-                                         len(self.action_q.content_queue),
-                                         len(self.hash_q),
-                                         self.fs.fs.cache_hits,
-                                         self.fs.fs.cache_misses, '-'*4))
+        """Log a "mark" that includes the current AQ state and queue size."""
+        self.logger.note("---- MARK (state: %s; queues: metadata: %d; content:"
+                         " %d; hash: %d, fsm-cache: hit=%d miss=%d) ----",
+                         self.state_manager, len(self.action_q.meta_queue),
+                         len(self.action_q.content_queue), len(self.hash_q),
+                         self.fs.fs.cache_hits, self.fs.fs.cache_misses)
 
     def wait_for_nirvana(self, last_event_interval=0.5):
-        """Get a deferred that will fire when there are no more
-        events or transfers."""
+        """Get a deferred that will fire on Nirvana.
+
+        That is, when there are no more events or transfers.
+        """
         self.logger.debug('wait_for_nirvana(%s)' % last_event_interval)
         d = defer.Deferred()
         def start():
-            """request the event empty notification"""
+            """Request the event empty notification."""
             self.logger.debug('starting wait_for_nirvana')
             self.event_q.add_empty_event_queue_callback(callback)
         def callback():
-            """event queue is empty"""
+            """Event queue is empty."""
             if not (self.state_manager.state == StateManager.QUEUE_MANAGER
                     and self.state_manager.queues.state == QueueManager.IDLE
                     and not self.action_q.meta_queue
                     and not self.action_q.content_queue
                     and self.hash_q.empty()):
-                self.logger.debug("I can't attain Nirvana yet."
-                                  " [state: %s; queues:"
-                                  " metadata: %d; content: %d; hash: %d]"
-                                  % (self.state_manager,
-                                     len(self.action_q.meta_queue),
-                                     len(self.action_q.content_queue),
-                                     len(self.hash_q)))
+                self.logger.debug("I can't attain Nirvana yet. [state: %s; "
+                                  "metadata: %d; content: %d; hash: %d]",
+                                  self.state_manager,
+                                  len(self.action_q.meta_queue),
+                                  len(self.action_q.content_queue),
+                                  len(self.hash_q))
                 return
             self.logger.debug("Nirvana reached!! I'm a Buddha")
             self.event_q.remove_empty_event_queue_callback(callback)
@@ -197,7 +183,7 @@ class Main(object):
                                      waiting_kwargs).deferred
 
     def start(self):
-        """setup the daemon to be ready to run"""
+        """Setup the daemon to be ready to run."""
         # hook the event queue to the root dir
         self.event_q.push('SYS_INIT_DONE')
         self.event_q.inotify_add_watch(self.root_dir)
@@ -223,8 +209,8 @@ class Main(object):
         self.action_q.check_version()
 
     def authenticate(self):
-        """Do the OAuth dance."""
-        self.action_q.authenticate(self.oauth_client.consumer)
+        """Do the authentication dance."""
+        self.action_q.authenticate()
 
     def set_capabilities(self):
         """Set the capabilities with the server"""
@@ -233,20 +219,7 @@ class Main(object):
 
     def server_rescan(self):
         """Do the server rescan."""
-        mdobj = self.fs.get_by_path(self.root_dir)
-        self.action_q.server_rescan(mdobj.mdid,
-                                    self.fs.get_data_for_server_rescan)
-
-    def set_oauth_token(self, key, secret):
-        """ Sets the oauth token """
-        self.token = oauth.OAuthToken(key, secret)
-
-    def get_access_token(self):
-        """Return the access token or a new one."""
-        if self.token:
-            return self.token
-        else:
-            return self.oauth_client.get_access_token()
+        return self.vm.server_rescan()
 
     def get_rootdir(self):
         """Return the base dir/mount point."""
@@ -287,36 +260,3 @@ class Main(object):
             self.event_q.push('SYS_UNKNOWN_ERROR')
 
         d.addCallbacks(local_rescan_done, stop_the_press)
-
-
-class NoAccessToken(Exception):
-    """No access token available."""
-
-
-class OAuthClient(object):
-    """ Basic OAuth client, just grab the token from the keyring. """
-
-    def __init__(self, realm, consumer_key='ubuntuone',
-                 consumer_secret='hammertime'):
-        """ create the instance and setup the modules """
-        self.realm = realm
-        self.consumer = oauth.OAuthConsumer(consumer_key, consumer_secret)
-
-    def get_access_token(self):
-        """Get the access token from the keyring.
-
-        If no token is available in the keyring, `NoAccessToken` is raised.
-        """
-        try:
-            items = gnomekeyring.find_items_sync(
-                gnomekeyring.ITEM_GENERIC_SECRET,
-                {'ubuntuone-realm': self.realm,
-                 'oauth-consumer-key': self.consumer.key})
-            return oauth.OAuthToken.from_string(items[0].secret)
-        except gnomekeyring.NoMatchError:
-            raise NoAccessToken("No access token found.")
-        except (gnomekeyring.NoKeyringDaemonError,
-                gnomekeyring.DeniedError), e:
-            raise NoAccessToken("Error while getting the token: %s" % type(e))
-
-
