@@ -20,28 +20,26 @@ from __future__ import with_statement
 
 import os
 import logging
+from operator import attrgetter
 from cStringIO import StringIO
 import sys
 
 from ubuntuone.syncdaemon.marker import MDMarker
-from ubuntuone.storageprotocol.dircontent_pb2 import DIRECTORY
-from ubuntuone.storageprotocol import dircontent
+from ubuntuone.storageprotocol import delta
 from ubuntuone.syncdaemon.fsm.fsm import \
     StateMachineRunner, StateMachine
-from ubuntuone.syncdaemon.interfaces import IMarker
 from ubuntuone.syncdaemon import u1fsfsm
 from ubuntuone.syncdaemon.logger import DebugCapture
-from ubuntuone.syncdaemon.filesystem_manager import \
-    InconsistencyError
+from ubuntuone.syncdaemon.filesystem_manager import InconsistencyError
+from ubuntuone.syncdaemon.volume_manager import VolumeDoesNotExist
+
 empty_hash = ""
 
 class FSKey(object):
-    """This encapsulates the problem of getting the metadata with different
-    keys."""
+    """Encapsulate the problem of getting the metadata with different keys."""
     __slots__ = ('fs', 'keys', 'mdid', '_changes')
 
     def __init__(self, fs, **keys):
-        """create"""
         self.fs = fs
         self.keys = keys
         self.mdid = None
@@ -98,7 +96,7 @@ class FSKey(object):
         self._changes.update(kwargs)
 
     def sync(self):
-        """sync the changes back to FSM"""
+        """Sync the changes back to FSM."""
         if self._changes and self.has_metadata():
             self.fs.set_by_mdid(self.get_mdid(), **self._changes)
             self._changes = {}
@@ -135,7 +133,7 @@ class FSKey(object):
             return 'NA'
 
     def open_file(self):
-        """get the file object for reading"""
+        """Get the file object for reading."""
         mdid = self.get_mdid()
         try:
             fo = self.fs.open_file(mdid)
@@ -148,34 +146,34 @@ class FSKey(object):
         return fo
 
     def upload_finished(self, server_hash):
-        """signal that we have uploaded the file"""
+        """Signal that we have uploaded the file."""
         mdid = self.get_mdid()
         self.fs.upload_finished(mdid, server_hash)
 
     def delete_file(self):
-        """delete the file and metadata"""
+        """Delete the file and metadata."""
         path = self["path"]
         self.fs.delete_file(path)
         self.mdid = None
         self._changes = {}
 
     def delete_to_trash(self):
-        """Move the node to trash"""
+        """Move the node to trash."""
         self.fs.delete_to_trash(self.get_mdid(), self["parent_id"])
 
     def remove_from_trash(self, share_id, node_id):
-        """Remove the node from trash"""
+        """Remove the node from trash."""
         self.fs.remove_from_trash(share_id, node_id)
 
     def delete_metadata(self):
-        """delete the metadata"""
+        """Delete the metadata."""
         path = self["path"]
         self.fs.delete_metadata(path)
         self.mdid = None
         self._changes = {}
 
     def move_file(self, new_share_id, new_parent_id, new_name):
-        """get the stuff we need to move the file."""
+        """Get the stuff we need to move the file."""
         source_path = self['path']
         parent_path = self.fs.get_by_node_id(new_share_id, new_parent_id).path
         dest_path = os.path.join(
@@ -184,13 +182,13 @@ class FSKey(object):
         self.fs.move_file(new_share_id, source_path, dest_path)
 
     def moved(self, new_share_id, path_to):
-        """change the metadata of a moved file."""
+        """Change the metadata of a moved file."""
         self.fs.moved(new_share_id, self['path'], path_to)
         if "path" in self.keys:
             self.keys["path"] = path_to
 
     def remove_partial(self):
-        """remove a partial file"""
+        """Remove a partial file."""
         # pylint: disable-msg=W0704
         try:
             self.fs.remove_partial(self["node_id"], self["share_id"])
@@ -199,11 +197,11 @@ class FSKey(object):
             pass
 
     def move_to_conflict(self):
-        """Move file to conflict"""
+        """Move file to conflict."""
         self.fs.move_to_conflict(self.get_mdid())
 
     def refresh_stat(self):
-        """refresh the stat"""
+        """Refresh the stat."""
         path = self["path"]
         # pylint: disable-msg=W0704
         try:
@@ -213,7 +211,7 @@ class FSKey(object):
             pass
 
     def safe_get(self, key, default='^_^'):
-        """ safe version of self.get, to be used in the FileLogger. """
+        """Safe version of self.get, to be used in the FileLogger."""
         # catch all errors as we are here to help logging
         # pylint: disable-msg=W0703
         try:
@@ -289,6 +287,7 @@ class FileLogger(object):
     debug = loglevel(logging.DEBUG)
     exception = loglevel(-1)
 
+
 class SyncStateMachineRunner(StateMachineRunner):
     """This is where all the state machine methods are."""
 
@@ -308,6 +307,7 @@ class SyncStateMachineRunner(StateMachineRunner):
         with DebugCapture(self.log.logger):
             func_name = super(SyncStateMachineRunner, self).on_event(*args,
                                                                      **kwargs)
+
             if not is_debug:
                 self.log.info("Called %s (In: %s)" % (func_name, in_state))
 
@@ -321,8 +321,9 @@ class SyncStateMachineRunner(StateMachineRunner):
         if newstat.st_ino != oldstat.st_ino or \
            newstat.st_size != oldstat.st_size or \
            newstat.st_mtime != oldstat.st_mtime:
-            m = "The received information is obsolete! New stat: %s"
-            self.log.debug(m, newstat)
+            self.log.debug("The received information is obsolete! New stat: "
+                           "st_ino=%d  st_size=%d  st_mtime=%r",
+                           newstat.st_ino, newstat.st_size, newstat.st_mtime)
             return False
         return True
 
@@ -341,10 +342,10 @@ class SyncStateMachineRunner(StateMachineRunner):
         params.update(self.build_hash_eq(hash))
         self.on_event(event, params, error, hash, *args)
 
-    def signal_event_with_error(self, event, error, *args):
+    def signal_event_with_error(self, event, failure, *args):
         """An event returned with error."""
-        params = self.build_error_eq(error)
-        self.on_event(event, params, error, *args)
+        params = self.build_error_eq(failure.getErrorMessage())
+        self.on_event(event, params, failure, *args)
 
     def build_error_eq(self, error):
         """Get the error state."""
@@ -360,6 +361,47 @@ class SyncStateMachineRunner(StateMachineRunner):
             is_directory=self.key.is_directory(),
         )
 
+    def update_generation(self, volume_id, node_id, new_generation):
+        """Update the generation for the node and volume."""
+
+        # update the file
+        try:
+            node = self.m.fs.get_by_node_id(volume_id, node_id)
+        except KeyError:
+            pass
+        else:
+            self.m.fs.set_by_mdid(node.mdid, generation=new_generation)
+
+        # update the volume
+        try:
+            volume = self.m.vm.get_volume(volume_id)
+        except VolumeDoesNotExist:
+            self.log.warning("Volume not found: %r", volume_id)
+            return
+
+        if volume.generation is None or new_generation is None:
+            self.log.debug("Client not ready for generations! vol gen: %r, "
+                             "new gen: %r", volume.generation, new_generation)
+            return
+
+        if new_generation <= volume.generation:
+            self.log.info("Got smaller or equal generation (%d) than current "
+                          "(%d) for volume %r", new_generation,
+                          volume.generation, volume_id)
+
+        elif new_generation == volume.generation + 1:
+            # last change from ourselves was the responsible of this increment
+            self.log.info("Updating current generation for volume %s to %d",
+                          volume_id, new_generation)
+            self.m.vm.update_generation(volume_id, new_generation)
+
+        else:
+            # there're changes that we don't know about, let's get a delta
+            m = "Generation much bigger (%d) than current (%d) for volume %r"
+            self.log.info(m, new_generation, volume.generation, volume_id)
+            self.m.action_q.get_delta(volume_id, volume.generation)
+
+
     # EVENT HANDLERS
 
     def nothing(self, event, params, *args):
@@ -370,28 +412,8 @@ class SyncStateMachineRunner(StateMachineRunner):
         """create a local file."""
         mdobj = self.m.fs.get_by_node_id(share_id, parent_id)
         path = os.path.join(self.m.fs.get_abspath(share_id, mdobj.path), name)
-        self.m.fs.create(path=path, share_id=share_id, node_id=node_id,
-                         is_dir=True)
-        self.m.action_q.query([(share_id, node_id, "")])
-        # pylint: disable-msg=W0704
-        # this should be provided by FSM, fix!!
-        try:
-            # pylint: disable-msg=W0212
-            with self.m.fs._enable_share_write(share_id, self.key['path']):
-                os.mkdir(self.key['path'])
-        except OSError, e:
-            if not e.errno == 17: #already exists
-                raise
-        else:
-            # just add the watch
-            # we hope the user wont have time to add a file just after
-            # *we* created the directory
-            # this is until we can solve and issue with LR and
-            # new dirs and fast downloads
-            # see bug #373940
-            self.m.event_q.inotify_add_watch(path)
-            #self.m.lr.scan_dir(path)
-
+        mdid = self.m.fs.create(path, share_id, node_id, True)
+        self.m.fs.make_dir(mdid)
 
     def new_dir_on_server_with_local_file(self, event, params, share_id,
                                           node_id, parent_id, name):
@@ -406,30 +428,6 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.m.fs.set_node_id(self.key['path'], node_id)
         self.reget_dir(event, params)
 
-    def reget_dir(self, event, params, hash=None):
-        """Reget the directory."""
-        self.m.action_q.cancel_download(share_id=self.key['share_id'],
-                            node_id=self.key['node_id'])
-        self.key.remove_partial()
-        self.m.action_q.query([(self.key['share_id'],
-                                self.key['node_id'],
-                                self.key['local_hash'] or "")])
-        self.key.set(server_hash=self.key['local_hash'])
-        self.key.sync()
-
-    def get_dir(self, event, params, hash):
-        """Get the directory."""
-        self.key.set(server_hash=hash)
-        self.key.sync()
-        self.m.fs.create_partial(node_id=self.key['node_id'],
-                                 share_id=self.key['share_id'])
-        self.m.action_q.listdir(
-            self.key['share_id'], self.key['node_id'], hash,
-            lambda : self.m.fs.get_partial_for_writing(
-                node_id=self.key['node_id'],
-                share_id=self.key['share_id'])
-            )
-
     def file_conflict(self, event, params, hash, crc32, size, stat):
         """This file is in conflict."""
         self.key.move_to_conflict()
@@ -441,92 +439,6 @@ class SyncStateMachineRunner(StateMachineRunner):
                             node_id=self.key['node_id'])
         self.get_file(event, params, hash)
 
-    def merge_directory(self, event, params, hash):
-        """Merge the server directory with the local one."""
-        new_files = []
-        new_dirs = []
-        deleted_filedirs = []
-        moved = set()
-
-        try:
-            fd = self.m.fs.get_partial(node_id=self.key['node_id'],
-                            share_id=self.key['share_id'])
-        except InconsistencyError:
-            self.key.remove_partial()
-            self.key.set(server_hash=self.key['local_hash'])
-            self.key.sync()
-            self.m.action_q.query([
-                (self.key["share_id"], self.key["node_id"], "")])
-            # we dont perform the merge, we try to re get it
-            return
-
-
-        items = dircontent.parse_dir_content(fd)
-        server_dir = [ (o.utf8_name, o.node_type == DIRECTORY, o.uuid)
-                        for o in items ]
-        client_dir = self.m.fs.dir_content(self.key['path'])
-        # XXX: lucio.torre: with huge dirs, this could take a while
-
-        share = self.key['share_id']
-        for name, isdir, uuid in server_dir:
-            # we took the name as bytes already encoded in utf8
-            # directly from dircontent!
-            try:
-                md = self.m.fs.get_by_node_id(share, uuid)
-            except KeyError:
-                # not there, a new thing
-                if isdir:
-                    new_dirs.append((share, uuid, name))
-                else:
-                    new_files.append((share, uuid, name))
-                continue
-            mdpath = self.m.fs.get_abspath(md.share_id, md.path)
-            if mdpath != os.path.join(self.key['path'], name):
-                # this was moved, or maybe the server still didn't receive
-                # the move that happened here
-                if not self.m.action_q.node_is_with_queued_move(share, uuid):
-                    # mark as moved
-                    moved.add(uuid)
-                    # signal moved
-                    self.m.event_q.push("SV_MOVED",
-                        share_id=md.share_id, node_id=uuid,
-                        new_share_id=share, new_parent_id=self.key['node_id'],
-                        new_name=name)
-
-
-        for name, isdir, uuid in client_dir:
-            if uuid is None:
-                continue
-
-            if not (name, isdir, uuid) in server_dir:
-                # not there, a its gone on the server
-                if uuid in moved:
-                    # this was a move, dont delete
-                    continue
-                deleted_filedirs.append((share, uuid))
-
-        # these nodes are in process of being deleted from the server, so they
-        # are not exactly new locally (were already deleted here)
-        trash = set((share_id, node_id) for (share_id, node_id, _)
-                                        in self.m.fs.get_iter_trash())
-
-        parent_uuid = self.key['node_id']
-        for share, uuid in deleted_filedirs:
-            self.m.event_q.push("SV_FILE_DELETED",
-                                node_id=uuid, share_id=share)
-        for share, uuid, name in new_files:
-            if (share, uuid) not in trash:
-                self.m.event_q.push("SV_FILE_NEW", parent_id=parent_uuid,
-                                    node_id=uuid, share_id=share, name=name)
-        for share, uuid, name in new_dirs:
-            if (share, uuid) not in trash:
-                self.m.event_q.push("SV_DIR_NEW", parent_id=parent_uuid,
-                                    node_id=uuid, share_id=share, name=name)
-
-        self.key.remove_partial()
-        self.key.set(local_hash=hash)
-        self.key.sync()
-
     def new_file(self, event, params, share_id, node_id, parent_id, name):
         """create a local file."""
         mdobj = self.m.fs.get_by_node_id(share_id, parent_id)
@@ -536,7 +448,6 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.key.set(server_hash="")
         self.key.set(local_hash="")
         self.key.sync()
-        self.m.action_q.query([(share_id, node_id, "")])
 
     def new_file_on_server_with_local(self, event, params, share_id,
                                       node_id, parent_id, name):
@@ -577,20 +488,14 @@ class SyncStateMachineRunner(StateMachineRunner):
         new_name = os.path.basename(path_to)
         new_parent = FSKey(self.m.fs, path=new_path)
         new_parent_id = new_parent['node_id']
+        share_id = self.key['share_id']
+        node_id = self.key['node_id']
 
-        self.m.action_q.move(share_id=self.key['share_id'],
-            node_id=self.key['node_id'], old_parent_id=old_parent_id,
-            new_parent_id=new_parent_id, new_name=new_name)
-        self.key.moved(self.key['share_id'], path_to)
-
-        # this is cheating, we change the state of another node
-        if not IMarker.providedBy(old_parent_id):
-            share_id = self.key['share_id']
-            self.m.action_q.cancel_download(share_id, old_parent_id)
-            old_parent.remove_partial()
-            self.m.fs.set_by_node_id(old_parent_id, share_id,
-                                     server_hash="", local_hash="")
-            self.m.action_q.query([(share_id, old_parent_id, "")])
+        self.m.action_q.move(share_id, node_id, old_parent_id,
+                             new_parent_id, new_name)
+        self.m.fs.add_to_move_limbo(share_id, node_id, old_parent_id,
+                                    new_parent_id, new_name)
+        self.key.moved(share_id, path_to)
 
         # we only hash it if we're a file, not a directory
         if not self.key.is_dir():
@@ -607,21 +512,18 @@ class SyncStateMachineRunner(StateMachineRunner):
 
     def commit_file(self, event, params, hash):
         """commit the new content."""
+        key = self.key
         try:
-            self.m.fs.commit_partial(
-                            self.key['node_id'], self.key['share_id'], hash)
+            self.m.fs.commit_partial(key['node_id'], key['share_id'], hash)
         except InconsistencyError:
-            # someone or something broke out partials.
-            # start work to go to a good state
-            self.key.remove_partial()
-            self.key.set(server_hash=self.key['local_hash'])
-            self.key.sync()
-            self.m.action_q.query([
-                (self.key["share_id"], self.key["node_id"], "")])
+            # someone or something broke out partials, log in warning
+            # and LR will take care and converge in the future
+            self.log.warning("Lost .partial when commiting node! volume_id=%r "
+                             "node_id=%r", key['share_id'], key['node_id'])
+
 
     def new_local_file(self, event, parms, path):
         """a new local file was created"""
-        # XXX: lucio.torre: we should use markers here
         parent_path = os.path.dirname(path)
         parent = self.m.fs.get_by_path(parent_path)
         parent_id = parent.node_id or MDMarker(parent.mdid)
@@ -634,14 +536,23 @@ class SyncStateMachineRunner(StateMachineRunner):
         marker = MDMarker(self.key.get_mdid())
         self.m.action_q.make_file(share_id, parent_id, name, marker)
 
-    def new_local_file_created(self, event, parms, new_id):
-        """we got the server answer for the file creation."""
-        self.m.fs.set_node_id(self.key['path'], new_id)
+    def release_marker_ok(self, event, parms, new_id, marker):
+        """Release ok the received marker in AQ's DeferredMap."""
+        self.m.action_q.uuid_map.set(marker, new_id)
+        self.m.fs.dereference_ok_limbos(marker, new_id)
 
+    def release_marker_error(self, event, parms, failure, marker):
+        """Release with error the received marker in AQ's DeferredMap."""
+        self.m.action_q.uuid_map.err(marker, failure)
+        self.m.fs.dereference_err_limbos(marker)
+
+    def new_local_file_created(self, event, parms, new_id, marker):
+        """We got the server answer for the file creation."""
+        self.m.action_q.uuid_map.set(marker, new_id)
+        self.m.fs.set_node_id(self.key['path'], new_id)
 
     def new_local_dir(self, event, parms, path):
         """a new local dir was created"""
-        # XXX: lucio.torre: we should use markers here
         parent_path = os.path.dirname(path)
         parent = self.m.fs.get_by_path(parent_path)
         parent_id = parent.node_id or MDMarker(parent.mdid)
@@ -653,14 +564,11 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.m.action_q.make_dir(share_id, parent_id, name, marker)
         self.m.lr.scan_dir(mdid, path)
 
-    def new_local_dir_created(self, event, parms, new_id):
+    def new_local_dir_created(self, event, parms, new_id, marker):
         """Server answered that dir creation was ok."""
+        self.m.action_q.uuid_map.set(marker, new_id)
         self.m.fs.set_node_id(self.key['path'], new_id)
 
-        # query to get any updates in case dir was already there in server
-        self.m.action_q.query([(self.key['share_id'], self.key['node_id'],
-                                self.key['local_hash'])])
-
     def calculate_hash(self, event, params):
         """calculate the hash of this."""
         self.m.hash_q.insert(self.key['path'], self.key['mdid'])
@@ -768,24 +676,12 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.key.remove_partial()
         self.delete_file(event, params)
 
-    def file_not_created_remove(self, event, params, error):
-        """kill it"""
+    def filedir_error_in_creation(self, event, params, failure, marker):
+        """Move actual content to conflict, and delete the metadata."""
+        self.m.action_q.uuid_map.err(marker, failure)
         self.key.move_to_conflict()
         self.key.delete_metadata()
 
-    def dir_not_created_in_server(self, event, params, error):
-        """Re-get the dir if it was because already exist."""
-        if error == "ALREADY_EXISTS":
-            # delete metadata and query the father to converge
-            parent = self.m.fs.get_by_node_id(self.key['share_id'],
-                                              self.key['parent_id'])
-            self.key.delete_metadata()
-            self.m.action_q.query([(parent.share_id,
-                                    parent.node_id, parent.local_hash)])
-        else:
-            self.key.move_to_conflict()
-            self.key.delete_metadata()
-
     def delete_on_server(self, event, params, path):
         """local file was deleted."""
         self.m.action_q.unlink(self.key['share_id'],
@@ -826,6 +722,10 @@ class SyncStateMachineRunner(StateMachineRunner):
         """Remove the node from trash."""
         self.key.remove_from_trash(share_id, node_id)
 
+    def clean_move_limbo(self, event, params, share_id, node_id):
+        """Remove the node from move limbo."""
+        self.m.fs.remove_from_move_limbo(share_id, node_id)
+
     def server_moved(self, event, params, share_id, node_id,
                      new_share_id, new_parent_id, new_name):
         """file was moved on the server"""
@@ -857,9 +757,6 @@ class SyncStateMachineRunner(StateMachineRunner):
         self.key.remove_partial()
         self.key.set(server_hash=self.key['local_hash'])
         self.key.sync()
-        self.m.action_q.query([(self.key['share_id'],
-                                self.key['node_id'],
-                                self.key['local_hash'] or "")])
 
     # pylint: disable-msg=C0103
     def DESPAIR(self, event, params, *args, **kwargs):
@@ -885,23 +782,47 @@ class Sync(object):
     # XXX: lucio.torre:
     # this will need some refactoring once we handle more events
 
+    fsm = None
+
     def __init__(self, main):
         """create"""
         # XXX: verterok: add a custom Logger to lazyily build the LogRecord,
         # now that the DebugCapture is enabled
         self.logger = logging.getLogger('ubuntuone.SyncDaemon.sync')
-        self.fsm = StateMachine(u1fsfsm.state_machine)
+        self.broken_logger = logging.getLogger(
+                                            'ubuntuone.SyncDaemon.BrokenNodes')
+        if Sync.fsm is None:
+            Sync.fsm = StateMachine(u1fsfsm.state_machine)
         self.m = main
         self.m.event_q.subscribe(self)
 
-    def handle_SV_HASH_NEW(self, share_id, node_id, hash):
-        """on SV_HASH_NEW"""
+    def mark_node_as_dirty(self, volume_id, node_id):
+        """Mark the node as dirty, log, and send the event."""
+        try:
+            mdobj = self.m.fs.get_by_node_id(volume_id, node_id)
+        except KeyError:
+            # node was not created
+            path = None
+            mdid = None
+        else:
+            path = mdobj.path
+            mdid = mdobj.mdid
+            self.m.fs.set_by_mdid(mdobj.mdid, dirty=True)
+
+        # send the event
+        self.m.event_q.push('SYS_BROKEN_NODE', volume_id=volume_id,
+                            node_id=node_id, path=path, mdid=mdid)
+        m = "Broken node: volume_id=%r node_id=%r mdid=%r path=%r"
+        self.broken_logger.info(m, volume_id, node_id, mdid, path)
+
+    def _handle_SV_HASH_NEW(self, share_id, node_id, hash):
+        """on SV_HASH_NEW.  No longer called by EQ, only internally."""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.signal_event_with_hash("SV_HASH_NEW", hash)
 
-    def handle_SV_FILE_NEW(self, share_id, node_id, parent_id, name):
+    def _handle_SV_FILE_NEW(self, share_id, node_id, parent_id, name):
         """on SV_FILE_NEW"""
         parent = FSKey(self.m.fs, share_id=share_id, node_id=parent_id)
         path = os.path.join(parent["path"], name)
@@ -910,7 +831,7 @@ class Sync(object):
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("SV_FILE_NEW", {}, share_id, node_id, parent_id, name)
 
-    def handle_SV_DIR_NEW(self, share_id, node_id, parent_id, name):
+    def _handle_SV_DIR_NEW(self, share_id, node_id, parent_id, name):
         """on SV_DIR_NEW"""
         parent = FSKey(self.m.fs, share_id=share_id, node_id=parent_id)
         path = os.path.join(parent["path"], name)
@@ -919,8 +840,8 @@ class Sync(object):
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("SV_DIR_NEW", {}, share_id, node_id, parent_id, name)
 
-    def handle_SV_FILE_DELETED(self, share_id, node_id):
-        """on SV_FILE_DELETED"""
+    def _handle_SV_FILE_DELETED(self, share_id, node_id):
+        """on SV_FILE_DELETED. Not called by EQ anymore."""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
@@ -997,33 +918,35 @@ class Sync(object):
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("FS_DIR_MOVE", {}, path_from, path_to)
 
-    def handle_AQ_FILE_NEW_OK(self, marker, new_id):
-        """on AQ_FILE_NEW_OK"""
+    def handle_AQ_FILE_NEW_OK(self, volume_id, marker, new_id, new_generation):
+        """On AQ_FILE_NEW_OK."""
         key = FSKey(self.m.fs, mdid=marker)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
-        ssmr.on_event("AQ_FILE_NEW_OK", {}, new_id)
+        ssmr.on_event("AQ_FILE_NEW_OK", {}, new_id, marker)
+        ssmr.update_generation(volume_id, new_id, new_generation)
 
-    def handle_AQ_FILE_NEW_ERROR(self, marker, error):
+    def handle_AQ_FILE_NEW_ERROR(self, marker, failure):
         """on AQ_FILE_NEW_ERROR"""
         key = FSKey(self.m.fs, mdid=marker)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
-        ssmr.signal_event_with_error("AQ_FILE_NEW_ERROR", error)
+        ssmr.signal_event_with_error("AQ_FILE_NEW_ERROR", failure, marker)
 
-    def handle_AQ_DIR_NEW_ERROR(self, marker, error):
+    def handle_AQ_DIR_NEW_ERROR(self, marker, failure):
         """on AQ_DIR_NEW_ERROR"""
         key = FSKey(self.m.fs, mdid=marker)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
-        ssmr.signal_event_with_error("AQ_DIR_NEW_ERROR", error)
+        ssmr.signal_event_with_error("AQ_DIR_NEW_ERROR", failure, marker)
 
-    def handle_AQ_DIR_NEW_OK(self, marker, new_id):
-        """on AQ_DIR_NEW_OK"""
+    def handle_AQ_DIR_NEW_OK(self, volume_id, marker, new_id, new_generation):
+        """On AQ_DIR_NEW_OK."""
         key = FSKey(self.m.fs, mdid=marker)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
-        ssmr.on_event("AQ_DIR_NEW_OK", {}, new_id)
+        ssmr.on_event("AQ_DIR_NEW_OK", {}, new_id, marker)
+        ssmr.update_generation(volume_id, new_id, new_generation)
 
     def handle_FS_FILE_CLOSE_WRITE(self, path):
         """on FS_FILE_CLOSE_WRITE"""
@@ -1047,7 +970,7 @@ class Sync(object):
         ssmr.on_event('HQ_HASH_ERROR', {})
 
     def handle_HQ_HASH_NEW(self, path, hash, crc32, size, stat):
-        """on HQ_HASH_NEW"""
+        """on HQ_HASH_NEW."""
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
@@ -1063,12 +986,14 @@ class Sync(object):
                 ssmr.signal_event_with_hash("HQ_HASH_NEW", hash,
                                             crc32, size, stat)
 
-    def handle_AQ_UPLOAD_FINISHED(self, share_id, node_id, hash):
-        """on AQ_UPLOAD_FINISHED"""
+    def handle_AQ_UPLOAD_FINISHED(self, share_id, node_id, hash,
+                                  new_generation):
+        """On AQ_UPLOAD_FINISHED."""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.signal_event_with_hash("AQ_UPLOAD_FINISHED", hash)
+        ssmr.update_generation(share_id, node_id, new_generation)
 
     def handle_AQ_UPLOAD_ERROR(self, share_id, node_id, error, hash):
         """on AQ_UPLOAD_ERROR"""
@@ -1077,7 +1002,7 @@ class Sync(object):
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.signal_event_with_error_and_hash("AQ_UPLOAD_ERROR", error, hash)
 
-    def handle_SV_MOVED(self, share_id, node_id, new_share_id, new_parent_id,
+    def _handle_SV_MOVED(self, share_id, node_id, new_share_id, new_parent_id,
                         new_name):
         """on SV_MOVED"""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
@@ -1086,12 +1011,14 @@ class Sync(object):
         ssmr.on_event("SV_MOVED", {}, share_id, node_id, new_share_id,
                       new_parent_id, new_name)
 
-    def handle_AQ_UNLINK_OK(self, share_id, parent_id, node_id):
-        """on AQ_UNLINK_OK"""
+    def handle_AQ_UNLINK_OK(self, share_id, parent_id, node_id,
+                            new_generation):
+        """On AQ_UNLINK_OK."""
         key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
         log = FileLogger(self.logger, key)
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("AQ_UNLINK_OK", {}, share_id, node_id)
+        ssmr.update_generation(share_id, node_id, new_generation)
 
     def handle_AQ_UNLINK_ERROR(self, share_id, parent_id, node_id, error):
         """on AQ_UNLINK_ERROR"""
@@ -1100,3 +1027,175 @@ class Sync(object):
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("AQ_UNLINK_ERROR", {}, share_id, node_id)
 
+    def handle_AQ_MOVE_OK(self, share_id, node_id, new_generation):
+        """On AQ_MOVE_OK."""
+        key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
+        log = FileLogger(self.logger, key)
+        ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
+        ssmr.on_event("AQ_MOVE_OK", {}, share_id, node_id)
+        ssmr.update_generation(share_id, node_id, new_generation)
+
+    def handle_AQ_MOVE_ERROR(self, share_id, node_id, parent_id, error):
+        """On AQ_MOVE_ERROR."""
+        key = FSKey(self.m.fs, share_id=share_id, node_id=node_id)
+        log = FileLogger(self.logger, key)
+        ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
+        ssmr.on_event("AQ_MOVE_ERROR", {}, share_id, node_id)
+
+    def handle_AQ_DELTA_OK(self, volume_id, delta_content, end_generation,
+                           full, free_bytes):
+        """We got a new delta. Apply item by item of the delta.
+
+        Unlike other Sync methods that just defer to Sync State Machine Runner,
+        delta operations sync state in a different way. So this method has a
+        lot of logic that you will not see in the other event handlers.
+        """
+
+        self.logger.info(
+            "handle_AQ_DELTA_OK for volume %s. (%s items)",
+            volume_id, len(delta_content))
+
+        to_delete = []
+
+        for dt in delta_content:
+            # we only know how to update files
+            try:
+                if not isinstance(dt, delta.FileInfoDelta):
+                    continue
+                # Unicode boundary, convert dt.name
+                dt_name = dt.name.encode('utf-8')
+                # we only support files and directories
+                if dt.file_type == delta.DIRECTORY:
+                    is_dir = True
+                elif dt.file_type == delta.FILE:
+                    is_dir = False
+                else:
+                    self.logger.warn("Unknown file type: %r" , dt.file_type)
+                    continue
+
+                # if the node is dead, call the remove handler and forget
+                # about it
+                if not dt.is_live:
+                    to_delete.append(dt)
+                    self._handle_SV_FILE_DELETED(dt.share_id, dt.node_id)
+                    # nothing else should happen with this
+                    continue
+
+                # here we must call handlers for SV_HASH_NEW, SV_MOVED
+                try:
+                    node = self.m.fs.get_by_node_id(dt.share_id, dt.node_id)
+                except KeyError:
+                    # if we're deleting this node right now, just ignore it
+                    # from the received delta
+                    if self.m.fs.node_in_trash(dt.share_id, dt.node_id):
+                        continue
+
+                    # node not there, we must create it
+                    if is_dir:
+                        self._handle_SV_DIR_NEW(dt.share_id, dt.node_id,
+                                                dt.parent_id, dt_name)
+                    else:
+                        self._handle_SV_FILE_NEW(dt.share_id, dt.node_id,
+                                                 dt.parent_id, dt_name)
+                    node = self.m.fs.get_by_node_id(dt.share_id, dt.node_id)
+
+                # if the delta is older than the node, skip!
+                if node.generation > dt.generation:
+                    continue
+
+                # if the path changed, we have a move, notify it
+                node_parent_id = self.m.fs.get_by_path(
+                    self.m.fs.get_abspath(node.share_id,
+                                          os.path.dirname(node.path))
+                    ).node_id
+                node_name = os.path.basename(node.path)
+                if dt.parent_id != node_parent_id or \
+                        dt_name != node_name:
+                    # this was moved, or maybe the server still didn't receive
+                    # the move that happened here
+                    if not self.m.action_q.node_is_with_queued_move(
+                            dt.share_id, dt.node_id):
+                        # signal moved
+                        self._handle_SV_MOVED(
+                            share_id=node.share_id, node_id=node.node_id,
+                            new_share_id=dt.share_id,
+                            new_parent_id=dt.parent_id, new_name=dt_name)
+                    else:
+                        self.logger.info("Not calling _handle_SV_MOVED for "
+                                         "%s:%s due to pending move. "
+                                         "(old parent = %s, new_parent = %s "
+                                         "old_name = %s, new_name = %s)",
+                                         node.share_id, node.node_id,
+                                         node_parent_id, dt.parent_id,
+                                         node_name, dt_name)
+
+
+                # if its a dir, theres nothing else that we do with them except
+                # creating them.
+                # if its a file, we only care about the hash
+                if not is_dir:
+                    self._handle_SV_HASH_NEW(dt.share_id, dt.node_id,
+                                        dt.content_hash)
+
+                # node updated, update generation
+                self.m.fs.set_by_mdid(node.mdid, generation=dt.generation)
+            except Exception:
+                # we trap all exceptions so we can continue processing deltas
+                # even if something fails for one file.
+                # Let's log
+                self.logger.exception(
+                    "Node delta for %s:%s can't be applied.",
+                    dt.share_id, dt.node_id)
+
+                # And mark the node as dirty
+                self.mark_node_as_dirty(dt.share_id, dt.node_id)
+
+        self.m.vm.update_free_space(volume_id, free_bytes)
+        self.m.vm.update_generation(volume_id, end_generation)
+
+        if not full:
+            self.m.action_q.get_delta(volume_id, end_generation)
+
+        self.logger.info(
+            "handle_AQ_DELTA_OK for volume %r done at generation %s. "
+            "(%s deletes)",
+            volume_id, end_generation, len(to_delete))
+
+    def handle_AQ_RESCAN_FROM_SCRATCH_OK(self, volume_id, delta_content,
+                                         end_generation, free_bytes):
+        """Handle rescan from scratch."""
+
+        self.logger.info(
+            "Applying rescan from scratch for volume %s. (%s items)",
+            volume_id, len(delta_content))
+
+        # remove root and apply the delta
+        new_delta_content = [ dt for dt in delta_content if dt.parent_id ]
+
+        self.handle_AQ_DELTA_OK(volume_id, new_delta_content, end_generation,
+                                True, free_bytes)
+
+        self.logger.info(
+            "Rescan from scratch apply delta for volume %s done.",
+            volume_id)
+        # get all nodes sorted by path and reversed
+        live_nodes = set(dt.node_id for dt in delta_content)
+        all_nodes = list(self.m.fs.get_mdobjs_by_share_id(volume_id))
+        all_nodes.sort(key=attrgetter("path"), reverse=True)
+
+        # remove all nodes not in the delta
+        # skip nodes that dont yet have node id, they are in process
+        deletes = 0
+        for node in all_nodes:
+            node_id = node.node_id
+
+            if node_id is None:
+                continue
+
+            if not node_id in live_nodes:
+                self._handle_SV_FILE_DELETED(volume_id, node_id)
+                deletes += 1
+
+        self.logger.info(
+            "Rescan from scratch for volume %s done. (%s deletes)",
+            volume_id, deletes)
