@@ -86,13 +86,17 @@ class _UDF(object):
 class Volume(object):
     """A generic volume."""
 
-    def __init__(self, volume_id, node_id):
+    # default generation for all volumes without a value in the metadata
+    generation = None
+
+    def __init__(self, volume_id, node_id, generation=None):
         """Create the volume."""
         # id and node_id should be str or None
         assert isinstance(volume_id, basestring) or volume_id is None
         assert isinstance(node_id, basestring) or node_id is None
         self.volume_id = volume_id
         self.node_id = node_id
+        self.generation = generation
 
     @property
     def id(self):
@@ -106,15 +110,19 @@ class Volume(object):
                   self.node_id == other.node_id)
         return result
 
+    def __repr__(self):
+        return "<Volume id %r, node_id %r, generation %r>" % (self.volume_id,
+                                                              self.node_id,
+                                                              self.generation)
 
 class Share(Volume):
     """A volume representing a Share."""
 
     def __init__(self, volume_id=None, node_id=None, path=None, name=None,
                  other_username=None, other_visible_name=None, accepted=False,
-                 access_level='View', free_bytes=None):
+                 access_level='View', free_bytes=None, generation=None):
         """Create the share."""
-        super(Share, self).__init__(volume_id, node_id)
+        super(Share, self).__init__(volume_id, node_id, generation)
         self.__dict__['type'] = 'Share'
         if path is None:
             self.path = None
@@ -133,10 +141,13 @@ class Share(Volume):
 
         The received path should be 'bytes'
         """
-        share = cls(str(share_response.id), str(share_response.subtree),
-                    path, share_response.name, share_response.other_username,
-                    share_response.other_visible_name,
-                    share_response.accepted, share_response.access_level)
+        share = cls(volume_id=str(share_response.id),
+                    node_id=str(share_response.subtree),
+                    path=path, name=share_response.name,
+                    other_username=share_response.other_username,
+                    other_visible_name=share_response.other_visible_name,
+                    accepted=share_response.accepted,
+                    access_level=share_response.access_level)
         return share
 
     @classmethod
@@ -160,11 +171,15 @@ class Share(Volume):
         The received path should be 'bytes'
 
         """
-        share = cls(str(share_volume.volume_id), str(share_volume.node_id),
-                    path, share_volume.share_name,
-                    share_volume.other_username,
-                    share_volume.other_visible_name, share_volume.accepted,
-                    share_volume.access_level)
+        share = cls(volume_id=str(share_volume.volume_id),
+                    node_id=str(share_volume.node_id),
+                    path=path, name=share_volume.share_name,
+                    other_username=share_volume.other_username,
+                    other_visible_name=share_volume.other_visible_name,
+                    accepted=share_volume.accepted,
+                    access_level=share_volume.access_level,
+                    generation=share_volume.generation,
+                    free_bytes=share_volume.free_bytes)
         return share
 
     def can_write(self):
@@ -188,6 +203,9 @@ class Share(Volume):
                   self.access_level == other.access_level)
         return result
 
+    def __repr__(self):
+        return "<Share id %r, node_id %r, generation %r, active %r>" % \
+                (self.volume_id, self.node_id, self.generation, self.active)
 
 class Shared(Share):
 
@@ -195,13 +213,17 @@ class Shared(Share):
         super(Shared, self).__init__(*args, **kwargs)
         self.__dict__['type'] = 'Shared'
 
+    def __repr__(self):
+        return "<Shared id %r, node_id %r>" % (self.volume_id, self.node_id)
+
 
 class Root(Volume):
     """A volume representing the root."""
 
-    def __init__(self, node_id=None, path=None, free_bytes=None):
+    def __init__(self, volume_id=request.ROOT, node_id=None, path=None,
+                 free_bytes=None, generation=None):
         """Create the Root."""
-        super(Root, self).__init__(request.ROOT, node_id)
+        super(Root, self).__init__(volume_id, node_id, generation)
         self.__dict__['type'] = 'Root'
         self.path = path
         self.free_bytes = free_bytes
@@ -218,14 +240,23 @@ class Root(Volume):
     def active(self):
         return True
 
+    @classmethod
+    def from_volume(cls, volume):
+        """Create a Root instance from a RootVolume."""
+        # TODO: include the generation and the volume_id(?)
+        return cls(node_id=str(volume.node_id), free_bytes=volume.free_bytes, generation=volume.generation)
+
+    def __repr__(self):
+        return "<Root node_id %r, generation %r>" % (self.node_id, self.generation)
+
 
 class UDF(Volume):
     """A volume representing a User Defined Folder."""
 
-    def __init__(self, volume_id=None, node_id=None,
-                 suggested_path=None, path=None, subscribed=True):
+    def __init__(self, volume_id=None, node_id=None, suggested_path=None,
+                 path=None, subscribed=True, generation=None):
         """Create the UDF, subscribed by default"""
-        super(UDF, self).__init__(volume_id, node_id)
+        super(UDF, self).__init__(volume_id, node_id, generation)
         self.__dict__['type'] = 'UDF'
         self.node_id = node_id
         self.suggested_path = suggested_path
@@ -233,7 +264,7 @@ class UDF(Volume):
         self.subscribed = subscribed
 
     def __repr__(self):
-        return "<UDF id %r, real path %r>" % (self.id, self.path)
+        return "<UDF id %r, generation %r, real path %r>" % (self.id, self.generation, self.path)
 
     @property
     def ancestors(self):
@@ -265,8 +296,10 @@ class UDF(Volume):
         The received path should be 'bytes'
 
         """
-        return cls(str(udf_volume.volume_id), str(udf_volume.node_id),
-                   udf_volume.suggested_path, path)
+        return cls(volume_id=str(udf_volume.volume_id),
+                   node_id=str(udf_volume.node_id),
+                   suggested_path=udf_volume.suggested_path, path=path,
+                   generation=udf_volume.generation)
 
     def __eq__(self, other):
         result = (super(UDF, self).__eq__(other) and
@@ -345,50 +378,51 @@ class VolumeManager(object):
         self.shares = VMFileShelf(self._shares_dir)
         self.shared = VMFileShelf(self._shared_dir)
         self.udfs = VMFileShelf(self._udfs_dir)
-        if self.shares.get(request.ROOT) is None:
-            self.root = Root(path=self.m.root_dir)
-        else:
-            self.root = self.shares[request.ROOT]
-        self.root.access_level = 'Modify'
-        self.root.path = self.m.root_dir
-        self.shares[request.ROOT] = self.root
+        root = self.shares.get(request.ROOT)
+        if root is None:
+            self.shares[request.ROOT] = root = Root(path=self.m.root_dir)
+        elif root.path != self.m.root_dir:
+            root.path = self.m.root_dir
+            self.shares[request.ROOT] = root
         self.marker_share_map = {}
         self.marker_udf_map = {}
         self.list_shares_retries = 0
         self.list_volumes_retries = 0
         self.retries_limit = 5
 
+    @property
+    def root(self):
+        """Return the Root share instance."""
+        return self.shares.get(request.ROOT)
+
     def init_root(self):
-        """ Creates the root mdid. """
+        """Creates the root mdid."""
         self.log.debug('init_root')
-        self._create_share_dir(self.root)
+        root = self.shares[request.ROOT]
+        self._create_share_dir(root)
         try:
-            self.m.fs.get_by_path(self.root.path)
+            self.m.fs.get_by_path(root.path)
         except KeyError:
-            self.m.fs.create(path=self.root.path,
-                             share_id=request.ROOT, is_dir=True)
+            self.m.fs.create(path=root.path,
+                             share_id=root.volume_id, is_dir=True)
 
-    def handle_SYS_ROOT_RECEIVED(self, root_id):
-        """Got the root, map it to the  root share."""
-        self.log.debug('handle_SYS_ROOT_RECEIVED(%s)', root_id)
-        self._set_root(root_id)
-        self.m.action_q.inquire_account_info()
-        self.m.action_q.inquire_free_space(request.ROOT)
-        self.refresh_volumes()
-        self.refresh_shares()
-
-    def _set_root(self, node_id):
+    def _got_root(self, node_id, free_bytes=None):
         """Set the root node_id to the root share and mdobj."""
         # only set the root if we don't have it
         root = self.shares[request.ROOT]
-        if not root.node_id:
-            mdobj = self.m.fs.get_by_path(self.root.path)
-            if getattr(mdobj, 'node_id', None) is None:
-                self.m.fs.set_node_id(self.root.path, node_id)
-            self.root.node_id = root.node_id = node_id
+        if free_bytes is not None:
+            root.free_bytes = free_bytes
+        mdobj = self.m.fs.get_by_path(root.path)
+        if not (root.node_id and mdobj.node_id):
+            self.m.fs.set_node_id(root.path, node_id)
+            root.node_id = node_id
             self.shares[request.ROOT] = root
+            self.m.event_q.push('SYS_ROOT_RECEIVED', root.node_id, mdobj.mdid)
         elif root.node_id != node_id:
             self.m.event_q.push('SYS_ROOT_MISMATCH', root.node_id, node_id)
+        else:
+            # the root node_id match and we already have it
+            self.m.event_q.push('SYS_ROOT_RECEIVED', root.node_id, mdobj.mdid)
 
     def refresh_shares(self):
         """Request the list of shares to the server."""
@@ -402,22 +436,140 @@ class VolumeManager(object):
         """Handle AQ_LIST_VOLUMES event."""
         self.log.debug('handling volumes list')
         self.list_volumes_retries = 0
-        shares = []
-        udfs = []
-        for volume in volumes:
-            vol = self._handle_new_volume(volume)
-            if isinstance(vol, Share):
-                shares.append(vol.id)
-            elif isinstance(vol, UDF):
-                udfs.append(vol.id)
+        shares, udfs = self._volumes_rescan_cb(volumes)
         self._cleanup_volumes(shares=shares, udfs=udfs)
 
-    def _handle_new_volume(self, volume):
-        """
-        Handle a (probably) new volume, and call the specific method to
-        handle this volume type (share/udf/root).
+    def handle_SV_VOLUME_NEW_GENERATION(self, volume_id, generation):
+        """Handle SV_VOLUME_NEW_GENERATION."""
+        self.log.debug('handle_SV_VOLUME_NEW_GENERATION(%r, %r)',
+                       volume_id, generation)
+        volume_id = str(volume_id) # be safe and use a str
+        try:
+            volume = self.get_volume(volume_id)
+        except VolumeDoesNotExist:
+            self.log.warning('Got a SV_VOLUME_NEW_GENERATION for a missing '
+                             'volume: %r with %r', volume_id, generation)
+            self.refresh_volumes()
+        else:
+            current_gen = volume.generation
+            if current_gen is None:
+                self.m.action_q.rescan_from_scratch(volume_id)
+            elif current_gen < generation:
+                # XXX: check if we want to impose a hard limit in the size of
+                # the delta and do a rescan from scratch if it's too big
+                self.m.action_q.get_delta(volume_id, current_gen)
+            elif current_gen >= generation:
+                self.log.info('Got SV_VOLUME_NEW_GENERATION(%r, %r) but volume'
+                              ' is at generation: %r',
+                              volume_id, generation, current_gen)
 
-        Returns the created share/udf.
+    def handle_AQ_DELTA_NOT_POSSIBLE(self, volume_id):
+        """Handle AQ_DELTA_NOT_POSSIBLE."""
+        self.log.debug('handle_AQ_DELTA_NOT_POSSIBLE(%r)', volume_id)
+        volume_id = str(volume_id)
+        try:
+            volume = self.get_volume(volume_id)
+        except VolumeDoesNotExist:
+            self.log.warning('Got a AQ_DELTA_NOT_POSSIBLE for a missing '
+                             'volume: %r', volume_id)
+            self.refresh_volumes()
+        else:
+            self.log.info('Requesting a rescan from scratch for: %s', volume)
+            self.m.action_q.rescan_from_scratch(volume_id)
+
+    def server_rescan(self):
+        """Do the 'server rescan'"""
+        d = self.m.action_q.query_volumes()
+        d.addCallback(self._volumes_rescan_cb)
+        def done(_):
+            self.m.event_q.push('SYS_SERVER_RESCAN_DONE')
+        def error(failure):
+            self.m.event_q.push('SYS_SERVER_RESCAN_ERROR',
+                                error=str(failure.value))
+        d.addCallbacks(done, error)
+        # refresh the pending shares and shared dirs list
+        d.addCallback(lambda _: self.refresh_shares())
+        return d
+
+    def _volumes_rescan_cb(self, volumes):
+        """Handle the volumes list for server rescan"""
+        self.log.debug('handling volumes rescan: %r', volumes)
+        events = []
+        shares = []
+        udfs = []
+        for new_volume in volumes:
+            # TODO: all this block might need to be inside a try:except
+            # to keep handling volumes even when one fails
+            volume_id = str(new_volume.volume_id or '')
+            try:
+                volume = self.get_volume(volume_id)
+            except VolumeDoesNotExist:
+                # a new volume!
+                self.log.debug('New volume! id: %r', volume_id)
+                volume = self._handle_new_volume(new_volume)
+                # if this new volume is a UDF, it will be skipped as
+                # isn't active until the local rescan finish.
+                # It's safe to skip it as a rescan_from_scratch is
+                # executed after the local rescan.
+            finally:
+                current_generation = volume.generation
+
+            # store the received volumes ids to return them
+            if isinstance(new_volume, RootVolume):
+                # handle root volume, to set root node_id
+                self._got_root(str(new_volume.node_id),
+                               free_bytes=new_volume.free_bytes)
+                shares.append(volume_id)
+                # don't check the fsm node, the root fsm object is created in init_root
+            elif isinstance(new_volume, ShareVolume):
+                # check if the fsm node exists, and create it if not
+                self._create_fsm_object(volume.path, volume.id, volume.node_id)
+                shares.append(volume_id)
+                # check if the accepted status changed
+                if volume.accepted != new_volume.accepted:
+                    volume.accepted = new_volume.accepted
+                    # update the share metadata
+                    self.shares[volume.volume_id] = volume
+                    # queue the VM_SHARE_CREATED event
+                    events.append(('VM_SHARE_CREATED',
+                                   dict(share_id=volume.volume_id)))
+            elif isinstance(new_volume, UDFVolume):
+                # check if the fsm node exists, and create it if not
+                self._create_fsm_object(volume.path, volume.id, volume.node_id)
+                udfs.append(volume_id)
+            else:
+                self.log.warning("Unknown type in the volumes list: %r",
+                                 volume)
+
+            # first check if it's active
+            if not volume.active:
+                self.log.info('Skipping inactive volume: %r', volume)
+                continue
+
+            new_generation = new_volume.generation
+            self.log.debug('New generation %s of %s', volume, new_generation)
+            if current_generation is None or \
+               current_generation < new_generation:
+                # add the event
+                d = dict(volume_id=volume_id, generation=new_generation)
+                events.append(('SV_VOLUME_NEW_GENERATION', d))
+
+            # update the free_bytes on the volume
+            self.update_free_space(volume_id, new_volume.free_bytes)
+
+        # push the collected events
+        for event in events:
+            self.m.event_q.push(event[0], **event[1])
+
+        return shares, udfs
+
+    def _handle_new_volume(self, volume):
+        """Handle a (probably) new volume.
+
+        Also call the specific method to handle this volume
+        type (share/udf/root).
+
+        Return the share/udf and create it if it doesn't exist.
 
         """
         if isinstance(volume, ShareVolume):
@@ -435,10 +587,10 @@ class VolumeManager(object):
             self.add_udf(udf)
             return udf
         elif isinstance(volume, RootVolume):
-            self._set_root(str(volume.node_id))
+            self._got_root(str(volume.node_id), free_bytes=volume.free_bytes)
+            return self.root
         else:
-            self.log.warning("Got a unkown type in the volmes list: %r",
-                             volume)
+            self.log.warning("Unknown type in the volumes list: %r", volume)
 
     def handle_AQ_SHARES_LIST(self, shares_list):
         """ handle AQ_SHARES_LIST event """
@@ -460,7 +612,9 @@ class VolumeManager(object):
                 self.add_share(share)
             elif a_share.direction == "from_me":
                 try:
-                    mdobj = self.m.fs.get_by_node_id("", a_share.subtree)
+                    mdobj = self.m.fs.get_by_node_id(
+                        str(a_share.subtree_volume_id or request.ROOT),
+                        str(a_share.subtree))
                     path = self.m.fs.get_abspath(mdobj.share_id, mdobj.path)
                 except KeyError:
                     # we don't have the file/md of this shared node_id yet
@@ -476,7 +630,7 @@ class VolumeManager(object):
 
     def _cleanup_volumes(self, shares=None, udfs=None):
         """Remove missing shares from the shares and shared shelfs."""
-        # housekeeping of the shares, shared and udfs shelf's each time we
+        # housekeeping of the shares, udfs shelf's each time we
         # get the list of shares/volumes
         self.log.debug('deleting dead volumes')
         if shares is not None:
@@ -541,19 +695,20 @@ class VolumeManager(object):
         # check AQ wait conditions
         self.m.action_q.check_conditions()
 
-    def handle_SV_SHARE_CHANGED(self, share_info):
+    def handle_SV_SHARE_CHANGED(self, info):
         """ handle SV_SHARE_CHANGED event """
-        if str(share_info.share_id) not in self.shares:
+        if str(info.share_id) not in self.shares:
             self.log.debug("New share notification, share_id: %s",
-                     share_info.share_id)
-            dir_name = self._build_share_path(share_info.share_name,
-                                              share_info.from_visible_name)
+                           info.share_id)
+            # XXX: request a refresh of the shares list(?)
+            dir_name = self._build_share_path(info.share_name,
+                                              info.from_visible_name)
             path = os.path.join(self.m.shares_dir, dir_name)
-            share = Share.from_notify_holder(share_info, path)
+            share = Share.from_notify_holder(info, path)
             self.add_share(share)
         else:
-            self.log.debug('share changed! %s', share_info.share_id)
-            self.share_changed(share_info)
+            self.log.debug('share changed! %s', info.share_id)
+            self.share_changed(info)
 
     def handle_SV_SHARE_DELETED(self, share_id):
         """ handle SV_SHARE_DELETED event """
@@ -579,6 +734,22 @@ class VolumeManager(object):
         if marker in self.marker_share_map:
             del self.marker_share_map[marker]
 
+    def handle_AQ_DELETE_SHARE_OK(self, share_id):
+        """Handle AQ_DELETE_SHARE_OK event."""
+        share = self.shared.get(share_id)
+        if share is not None:
+            del self.shared[share_id]
+            self.m.event_q.push('VM_SHARE_DELETED', share)
+        else:
+            # delete ok, of something we don't have.
+            self.log.warning("AQ_DELETE_SHARE_OK of a non-existing shared "
+                             "dir: %s", share_id)
+
+    def handle_AQ_DELETE_SHARE_ERROR(self, share_id, error):
+        """Gandle AQ_DELETE_SHARE_ERROR event."""
+        self.log.error("Error while deleting offered share: %s", error)
+        self.m.event_q.push('VM_SHARE_DELETE_ERROR', share_id, error)
+
     def handle_SV_SHARE_ANSWERED(self, share_id, answer):
         """ handle SV_SHARE_ANSWERED event. """
         share = self.shared.get(share_id, None)
@@ -592,30 +763,41 @@ class VolumeManager(object):
             self.shared[share_id] = share
 
     def handle_AQ_ANSWER_SHARE_OK(self, share_id, answer):
-        """ Handle successfully accepting a share """
+        """Handle successfully accepting a share."""
         if answer == 'Yes':
             share = self.shares[share_id]
+            share.accepted = True
+            self.shares[share.volume_id] = share
             self._create_fsm_object(share.path, share.volume_id, share.node_id)
             self._create_share_dir(share)
-            self.m.action_q.query([(share.volume_id, str(share.node_id), "")])
-            self.m.action_q.inquire_free_space(share.volume_id)
+            self.m.action_q.rescan_from_scratch(share.volume_id)
 
-    def add_share(self, share):
+    def add_share(self, a_share):
         """ Add a share to the share list, and creates the fs mdobj. """
         self.log.info('Adding new share with id: %s - path: %r',
-                      share.volume_id, share.path)
-        if share.volume_id in self.shares:
-            del self.shares[share.volume_id]
+                      a_share.volume_id, a_share.path)
+        share = self.shares.get(a_share.volume_id)
+        is_new_share = share is None
+        if share is not None:
+            share.accepted = a_share.accepted
+            self.shares[share.volume_id] = share
+        else:
+            share = a_share
+        if is_new_share or not share.accepted:
+            # if it's a new share or isn't accepted set the generation to None
+            # to force a rescan
+            share.generation = None
+        # store the share
         self.shares[share.volume_id] = share
         if share.accepted:
             self._create_fsm_object(share.path, share.volume_id, share.node_id)
             self._create_share_dir(share)
-            self.m.action_q.query([(share.volume_id, str(share.node_id), "")])
-            self.m.action_q.inquire_free_space(share.volume_id)
+        if is_new_share:
+            # push the event only if it's a new share
             self.m.event_q.push('VM_SHARE_CREATED', share.volume_id)
 
     def update_free_space(self, volume_id, free_bytes):
-        """ Update free space for a given share."""
+        """Update free space for a given share."""
         if volume_id in self.shares:
             share = self.shares[volume_id]
             share.free_bytes = free_bytes
@@ -627,6 +809,10 @@ class VolumeManager(object):
         else:
             self.log.warning("Update of free space requested, but there is "
                              "no such volume_id: %s", volume_id)
+            self.refresh_volumes()
+
+    # same functionality, but other name to be called by EventQueue
+    handle_SYS_QUOTA_EXCEEDED = update_free_space
 
     def get_free_space(self, volume_id):
         """Return the free_space for volume_id.
@@ -643,6 +829,7 @@ class VolumeManager(object):
         else:
             self.log.warning("Requested free space of volume_id: %s, but"
                              " there is no such volume.", volume_id)
+            self.refresh_volumes()
             return 0
 
     def accept_share(self, share_id, answer):
@@ -660,7 +847,7 @@ class VolumeManager(object):
         self.log.debug("Share (id: %s) deleted. ", share_id)
         share = self.shares.get(share_id, None)
         if share is None:
-            # we don't have this share, ignore it
+            # we don't have this share, ignore it and don't refresh
             self.log.warning("Got a share deleted notification (%r), "
                              "but don't have the share", share_id)
         else:
@@ -671,13 +858,14 @@ class VolumeManager(object):
             self.m.event_q.push('VM_VOLUME_DELETED', share)
 
     def share_changed(self, share_holder):
-        """ process the share changed event """
+        """Process the share changed event."""
         # the holder id is a uuid, use the str
         share = self.shares.get(str(share_holder.share_id), None)
         if share is None:
-            # we don't have this share, ignore it
+            # we don't have this share, refresh volumes
             self.log.warning("Got a share changed notification (%r), "
                              "but don't have the share", share_holder.share_id)
+            self.refresh_volumes()
         else:
             share.access_level = share_holder.access_level
             self.shares[share.volume_id] = share
@@ -744,7 +932,7 @@ class VolumeManager(object):
             node_id = marker
         else:
             node_id = mdobj.node_id
-        share = Shared(path=self.m.fs.get_abspath("", mdobj.path),
+        share = Shared(path=self.m.fs.get_abspath(mdobj.share_id, mdobj.path),
                       volume_id=marker,
                       name=name, access_level=access_level,
                       other_username=username, other_visible_name=None,
@@ -753,6 +941,15 @@ class VolumeManager(object):
         self.m.action_q.create_share(node_id, username, name,
                                      access_level, marker)
 
+    def delete_share(self, share_id):
+        """Reuqest the deletion of an offered share."""
+        share = self.shared.get(share_id)
+        if share is None:
+            # if the share isn't there, push the error!
+            self.m.event_q.push('VM_SHARE_DELETE_ERROR', share_id, "DOES_NOT_EXIST")
+        else:
+            self.m.action_q.delete_share(share_id)
+
     def add_shared(self, share):
         """ Add a share with direction == from_me """
         self.log.info('New shared subtree: id: %s - path: %r',
@@ -769,12 +966,15 @@ class VolumeManager(object):
         """Add the udf to the VM metadata if isn't there.
 
         If it's a new udf, create the directory, hook inotify
-        and execute a query.
+        and request the full delta.
 
         """
         self.log.debug('add_udf: %s', udf)
         if self.udfs.get(udf.volume_id, None) is None:
             self.log.debug('udf not in metadata, adding it!')
+            # if it's a new UDF set the generation to None to force a
+            # rescan
+            udf.generation = None
             self.udfs[udf.volume_id] = udf
             self._create_fsm_object(udf.path, udf.volume_id, udf.node_id)
             # local and server rescan, this will add the inotify hooks
@@ -950,6 +1150,7 @@ class VolumeManager(object):
             udf = self.udfs[udf_id]
         except KeyError:
             push_error("DOES_NOT_EXIST")
+            d = defer.fail(VolumeDoesNotExist(udf_id))
         else:
             if not os.path.exists(udf.path):
                 # the udf path isn't there, create it!
@@ -966,24 +1167,23 @@ class VolumeManager(object):
                 return result
             try:
                 d = self._scan_udf(udf)
-            except KeyError:
+            except KeyError, e:
                 push_error("METADATA_DOES_NOT_EXIST")
+                d = defer.fail(e)
             else:
                 d.addCallback(subscribe)
                 d.addCallbacks(
                     lambda _: self.m.event_q.push('VM_UDF_SUBSCRIBED', udf),
                     lambda f: push_error(f.getErrorMessage()))
-            return d
+        return d
 
     def _scan_udf(self, udf):
         """Local and server rescan of a UDF."""
         mdobj = self.m.fs.get_by_path(udf.path)
         d = self.m.lr.scan_dir(mdobj.mdid, udf.path, udfmode=True)
         def server_rescan(_):
-            """Do a query over all known nodes."""
-            data = self.m.fs.get_for_server_rescan_by_path(udf.path)
-            self.m.action_q.query(data)
-            self.m.action_q.inquire_free_space(request.ROOT)
+            """Request the delta from the last known generation."""
+            self.m.action_q.rescan_from_scratch(udf.volume_id)
         d.addCallback(server_rescan)
         return d
 
@@ -1023,11 +1223,15 @@ class VolumeManager(object):
         """Handle AQ_DELETE_VOLUME_ERROR."""
         try:
             self.get_volume(str(volume_id))
-        except KeyError:
-            self.log.warning("Received a AQ_DELETE_VOLUME_ERROR of a missing"
+        except VolumeDoesNotExist:
+            # wasn't able to delete a volume that we don't have any
+            # more, we better refresh everything
+            self.log.warning("Received a AQ_DELETE_VOLUME_ERROR of a missing "
                              "volume id")
+            self.refresh_volumes()
         else:
-            self.m.event_q.push('VM_VOLUME_DELETE_ERROR', volume_id, str(error))
+            self.m.event_q.push('VM_VOLUME_DELETE_ERROR',
+                                volume_id, str(error))
 
     def handle_SV_VOLUME_CREATED(self, volume):
         """Handle SV_VOLUME_CREATED event."""
@@ -1052,9 +1256,21 @@ class VolumeManager(object):
             self.log.debug('udf deleted! %s', volume.id)
             self.udf_deleted(volume.id)
         else:
+            # just log, don't care we don't have something we should
+            # delete anyway
             self.log.warning("Tried to delete a missing volume id: %s",
                              volume_id)
 
+    def update_generation(self, volume_id, generation):
+        """Update the generation of the specified volume."""
+        self.log.debug('update_generation: %r, %r', volume_id, generation)
+        vol = self.get_volume(volume_id)
+        vol.generation = generation
+        if isinstance(vol, (Share, Root)):
+            self.shares[volume_id] = vol
+        elif isinstance(vol, UDF):
+            self.udfs[volume_id] = vol
+
 
 class MetadataUpgrader(object):
     """A class that loads old metadata and migrate it."""
@@ -1385,7 +1601,6 @@ class MetadataUpgrader(object):
             if 'id' in share.__dict__:
                 share.volume_id = share.__dict__.pop('id')
             if 'free_bytes' in share.__dict__:
-                # FIXME: REVIEWERS PLEASE CONFIRM THIS IS CORRECT
                 share.free_bytes = share.__dict__.pop('free_bytes')
             else:
                 share.free_bytes = None
@@ -1399,7 +1614,8 @@ class MetadataUpgrader(object):
             return clazz(**share_dict)
         elif share.path == self._root_dir or share.id == '':
             # handle the root special case
-            return Root(share.subtree, share.path)
+            return Root(volume_id=request.ROOT,
+                        node_id=share.subtree, path=share.path)
         else:
             share = upgrade_share_dict(share)
             if shared:
