@@ -28,21 +28,18 @@ import shutil
 from Queue import Queue
 from threading import Lock
 import zlib
-import urlparse
-import ConfigParser
 from cStringIO import StringIO
 
 from twisted.internet import reactor, defer
 from twisted.internet.defer import inlineCallbacks, returnValue
-from ubuntuone.storageprotocol.hash import crc32
+from ubuntuone.logger import LOGFOLDER
+from ubuntuone.storageprotocol.content_hash import crc32
 from ubuntuone.storageprotocol.context import get_ssl_context
-from ubuntuone.oauthdesktop.config import get_config \
-                                                 as get_oauth_config
-from ubuntuone.oauthdesktop.auth import AuthorisationClient
 from ubuntuone.u1sync.genericmerge import MergeNode
 from ubuntuone.u1sync.utils import should_sync
 
 CONSUMER_KEY = "ubuntuone"
+CONSUMER_SECRET = "hammertime"
 
 from oauth.oauth import OAuthConsumer
 from ubuntuone.storageprotocol.client import (
@@ -53,15 +50,12 @@ from ubuntuone.storageprotocol.dircontent_pb2 import \
 import uuid
 import logging
 from logging.handlers import RotatingFileHandler
-import xdg.BaseDirectory
 import time
 
 def share_str(share_uuid):
     """Converts a share UUID to a form the protocol likes."""
     return str(share_uuid) if share_uuid is not None else request.ROOT
 
-log_home = xdg.BaseDirectory.xdg_cache_home
-LOGFOLDER = os.path.join(log_home, 'ubuntuone','log')
 LOGFILENAME = os.path.join(LOGFOLDER, 'u1sync.log')
 u1_logger = logging.getLogger("u1sync.timing.log")
 handler = RotatingFileHandler(LOGFILENAME)
@@ -180,9 +174,12 @@ class Client(object):
     """U1 storage client facade."""
     required_caps = frozenset(["no-content", "fix462230"])
 
-    def __init__(self, realm, reactor=reactor):
-        """Create the instance."""
+    def __init__(self, realm=None, reactor=reactor):
+        """Create the instance.
 
+        'realm' is no longer used, but is left as param for API compatibility.
+
+        """
         self.reactor = reactor
         self.factory = SyncClientFactory(self)
 
@@ -192,37 +189,8 @@ class Client(object):
         self._status_waiting = []
         self._active_waiters = set()
 
-        self.realm = realm
-
-        oauth_config = get_oauth_config()
-        if oauth_config.has_section(realm):
-            config_section = realm
-        elif self.realm.startswith("http://localhost") and \
-             oauth_config.has_section("http://localhost"):
-            config_section = "http://localhost"
-        else:
-            config_section = "default"
-
-        @log_timing
-        def get_oauth_option(option):
-            """Retrieves an option from oauth config."""
-            try:
-                return oauth_config.get(config_section, option)
-            except ConfigParser.NoOptionError:
-                return oauth_config.get("default", option)
-
-        @log_timing
-        def get_oauth_url(option):
-            """Retrieves an absolutized URL from the OAuth config."""
-            suffix = get_oauth_option(option)
-            return urlparse.urljoin(realm, suffix)
-
         self.consumer_key = CONSUMER_KEY
-        self.consumer_secret = get_oauth_option("consumer_secret")
-
-        self.request_token_url = get_oauth_url("request_token_url")
-        self.user_authorisation_url = get_oauth_url("user_authorisation_url")
-        self.access_token_url = get_oauth_url("access_token_url")
+        self.consumer_secret = CONSUMER_SECRET
 
     def force_shutdown(self):
         """Forces the client to shut itself down."""
@@ -258,49 +226,6 @@ class Client(object):
                 if waiter in self._active_waiters:
                     self._active_waiters.remove(waiter)
 
-    @log_timing
-    def obtain_oauth_token(self, create_token):
-        """Obtains an oauth token, optionally creating one if requried."""
-        token_result = self._get_waiter()
-
-        @log_timing
-        def have_token(token):
-            """When a token is available."""
-            token_result.wake(token)
-
-        @log_timing
-        def no_token():
-            """When no token is available."""
-            token_result.wake(None)
-
-        oauth_client = AuthorisationClient(realm=self.realm,
-                                           request_token_url=
-                                           self.request_token_url,
-                                           user_authorisation_url=
-                                           self.user_authorisation_url,
-                                           access_token_url=
-                                           self.access_token_url,
-                                           consumer_key=self.consumer_key,
-                                           consumer_secret=
-                                           self.consumer_secret,
-                                           callback_parent=have_token,
-                                           callback_denied=no_token,
-                                           do_login=create_token)
-
-
-        @log_timing
-        def _obtain_token():
-            """Obtains or creates a token."""
-            if create_token:
-                oauth_client.clear_token()
-            oauth_client.ensure_access_token()
-
-        self.reactor.callFromThread(_obtain_token)
-        token = self._wait(token_result)
-        if token is None:
-            raise AuthenticationError("Unable to obtain OAuth token.")
-        return token
-
     @log_timing
     def _change_status(self, status, reason=None):
         """Changes the client status.  Usually called from the reactor
@@ -415,10 +340,11 @@ class Client(object):
         self._await_status_not("connecting", "connected", "authenticated")
 
     @log_timing
-    def oauth_from_token(self, token):
+    def oauth_from_token(self, token, consumer=None):
         """Perform OAuth authorisation using an existing token."""
 
-        consumer = OAuthConsumer(self.consumer_key, self.consumer_secret)
+        if consumer is None:
+            consumer = OAuthConsumer(self.consumer_key, self.consumer_secret)
 
         def _auth_successful(value):
             """Callback for successful auth.  Changes status to
