@@ -35,7 +35,7 @@ gobject.set_application_name('u1sync')
 from twisted.internet import reactor
 from Queue import Queue
 
-from oauth.oauth import OAuthToken
+from oauth.oauth import OAuthToken, OAuthConsumer
 import ubuntuone.storageprotocol.dircontent_pb2 as dircontent_pb2
 from ubuntuone.storageprotocol.dircontent_pb2 import DIRECTORY, SYMLINK
 from ubuntuone.u1sync.genericmerge import (
@@ -282,7 +282,6 @@ def do_diff(client, share_spec, directory, quiet, subtree_path,
 def do_main(argv):
     """The main user-facing portion of the script."""
     usage = "Usage: %prog [options] [DIRECTORY]\n" \
-            "       %prog --authorize [options]\n" \
             "       %prog --list-shares [options]\n" \
             "       %prog --init [--share=SHARE_UUID] [options] DIRECTORY\n" \
             "       %prog --diff [--share=SHARE_UUID] [options] DIRECTORY"
@@ -293,13 +292,13 @@ def do_main(argv):
     parser.add_option("--host", dest="host", metavar="HOST",
                       default='fs-1.one.ubuntu.com',
                       help="The server address")
-    parser.add_option("--realm", dest="realm", metavar="REALM",
-                      default='https://ubuntuone.com',
-                      help="The oauth realm")
-    parser.add_option("--oauth", dest="oauth", metavar="KEY:SECRET",
-                      default=None,
-                      help="Explicitly provide OAuth credentials "
-                           "(default is to query keyring)")
+
+    oauth_meta_var = "[CONSUMER_KEY:CONSUMER_SECRET:]KEY:SECRET"
+    oauth_help = "Explicitly provide OAuth credentials. You can either " \
+                 "provide the 4 values (consumer key and secret, plus the " \
+                 "key and secret), or just the last two (key and secret)."
+    parser.add_option("--oauth", dest="oauth", metavar=oauth_meta_var,
+                      help=oauth_help)
     action_list = ", ".join(sorted(MERGE_ACTIONS.keys()))
     parser.add_option("--action", dest="action", metavar="ACTION",
                       default=None,
@@ -310,9 +309,6 @@ def do_main(argv):
                                           "making changes")
     parser.add_option("--quiet", action="store_true", dest="quiet",
                       default=False, help="Produces less output")
-    parser.add_option("--authorize", action="store_const", dest="mode",
-                      const="authorize",
-                      help="Authorize this machine")
     parser.add_option("--list-shares", action="store_const", dest="mode",
                       const="list-shares", default="sync",
                       help="List available shares")
@@ -354,17 +350,14 @@ def do_main(argv):
         else:
             directory = args[0]
     if options.mode == "init" or options.mode == "list-shares" or \
-       options.mode == "diff" or options.mode == "authorize":
+       options.mode == "diff":
         if options.action is not None:
             parser.error("--%s does not take the --action parameter" % \
                          options.mode)
         if options.dry_run:
             parser.error("--%s does not take the --dry-run parameter" % \
                          options.mode)
-    if options.mode == "authorize":
-        if options.oauth is not None:
-            parser.error("--authorize does not take the --oauth parameter")
-    if options.mode == "list-shares" or options.mode == "authorize":
+    if options.mode == "list-shares":
         if len(args) != 0:
             parser.error("--list-shares does not take a directory")
     if options.mode != "init" and options.mode != "diff":
@@ -379,15 +372,20 @@ def do_main(argv):
         options.action = DEFAULT_MERGE_ACTION
 
 
+    token = None
+    consumer = None
     if options.oauth is None:
-        passed_token = None
+        parser.error("--oauth is required")
     else:
-        try:
-            (key, secret) = options.oauth.split(':', 2)
-        except ValueError:
-            parser.error("--oauth requires a key and secret together in the "
-                         " form KEY:SECRET")
-        passed_token = OAuthToken(key, secret)
+        values = options.oauth.split(':')
+        if len(values) == 4:
+            (consumer_key, consumer_secret, key, secret) = values
+            consumer = OAuthConsumer(consumer_key, consumer_secret)
+        elif len(values) == 2:
+            (key, secret) = values
+        else:
+            parser.error("--oauth requires %s" % oauth_meta_var)
+        token = OAuthToken(key, secret)
 
     if options.share is not None:
         try:
@@ -398,24 +396,18 @@ def do_main(argv):
     else:
         share_spec = None
 
-    client = Client(realm=options.realm, reactor=reactor)
+    client = Client(reactor=reactor)
 
     signal.signal(signal.SIGINT, lambda s, f: client.force_shutdown())
     signal.signal(signal.SIGTERM, lambda s, f: client.force_shutdown())
 
     def run_client():
         """Run the blocking client."""
-        if passed_token is None:
-            should_create_token = (options.mode == "authorize")
-            token = client.obtain_oauth_token(create_token=should_create_token)
-        else:
-            token = passed_token
-
         client.connect_ssl(options.host, int(options.port), options.no_ssl_verify)
 
         try:
             client.set_capabilities()
-            client.oauth_from_token(token)
+            client.oauth_from_token(token, consumer)
 
             if options.mode == "sync":
                 do_sync(client=client, directory=directory,
@@ -432,9 +424,6 @@ def do_main(argv):
                         directory=directory,
                         quiet=options.quiet, subtree_path=options.subtree,
                         ignore_symlinks=False)
-            elif options.mode == "authorize":
-                if not options.quiet:
-                    print "Authorized."
         finally:
             client.disconnect()
 
